@@ -1,4 +1,4 @@
-/* SPDX-License-Identifier: GPL-2.0 */
+/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
 /* Copyright Authors of Cilium */
 
 #ifndef __DATA_MSG_
@@ -13,7 +13,9 @@ struct data_event_id {
 
 struct data_event_desc {
 	__s32 error;
+	__u32 pad;
 	__u32 leftover;
+	__u32 size;
 	struct data_event_id id;
 } __attribute__((packed));
 
