@@ -1,4 +1,4 @@
-// SPDX-License-Identifier: GPL-2.0
+// SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause)
 /* Copyright Authors of Cilium */
 
 #ifndef _MSG_TYPES_
@@ -12,6 +12,7 @@ enum msg_ops {
 	MSG_OP_GENERIC_KPROBE = 13,
 	MSG_OP_GENERIC_TRACEPOINT = 14,
 	MSG_OP_GENERIC_UPROBE = 15,
+	MSG_OP_GENERIC_LSM = 16,
 
 	MSG_OP_TEST = 254,
 
@@ -29,6 +30,10 @@ enum msg_ops {
 
 	MSG_OP_CGROUP = 25,
 
+	MSG_OP_LOADER = 26,
+
+	MSG_OP_THROTTLE = 27,
+
 	MSG_OP_MAX,
 };
 
