@@ -1,4 +1,4 @@
-// SPDX-License-Identifier: GPL-2.0
+// SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause)
 /* Copyright Authors of Cilium */
 
 #ifndef __OPERATIONS_H__
@@ -27,6 +27,22 @@ enum {
 	op_filter_sport = 15,
 	op_filter_dport = 16,
 	op_filter_protocol = 17,
+	op_filter_notsport = 18,
+	op_filter_notdport = 19,
+	op_filter_sportpriv = 20,
+	op_filter_notsportpriv = 21,
+	op_filter_dportpriv = 22,
+	op_filter_notdportpriv = 23,
+	op_filter_notsaddr = 24,
+	op_filter_notdaddr = 25,
+	// file ops
+	op_filter_str_notprefix = 26,
+	op_filter_str_notpostfix = 27,
+	// more socket ops
+	op_filter_family = 28,
+	op_filter_state = 29,
+	// capability ops
+	op_capabilities_gained = 30,
 };
 
 #endif // __OPERATIONS_H__
