@@ -4,9 +4,11 @@
 package main
 
 import (
+	"github.com/cilium/tetragon/cmd/tetra/loglevel"
 	"github.com/spf13/cobra"
 )
 
 func addCommands(rootCmd *cobra.Command) {
 	addBaseCommands(rootCmd)
+	rootCmd.AddCommand(loglevel.New())
 }
