@@ -1,3 +1,8 @@
+// SPDX-License-Identifier: Apache-2.0
+// Copyright Authors of Tetragon
+
+//go:build !windows
+
 package main
 
 import (
@@ -7,28 +12,28 @@ import (
 	"github.com/cilium/tetragon/pkg/vmtests"
 )
 
-// NB: we should use lvh's RunConf to avoid duplicating code
 type RunConf struct {
-	testImage             string
-	baseFname             string
-	kernelFname           string
+	runner.RunConf
+	vmName                string
+	baseImageFilename     string
 	dontRebuildImage      bool
 	useTetragonTesterInit bool
 	testerOut             string
 	qemuPrint             bool
 	justBoot              bool
 	justBuildImage        bool
-	disableKVM            bool
-	enableHVF             bool
 	btfFile               string
 	disableUnifiedCgroups bool
-	portForwards          runner.PortForwards
 	testerConf            vmtests.Conf
+	detailedResults       bool
+	keepAllLogs           bool
 
 	filesystems []QemuFS
 }
 
-func (rc *RunConf) testImageFname() string {
-	imagesDir := filepath.Dir(rc.baseFname)
-	return filepath.Join(imagesDir, rc.testImage)
+func (rc RunConf) testImageFilename() string {
+	if ext := filepath.Ext(rc.vmName); ext == "" {
+		return rc.vmName + ".qcow2"
+	}
+	return rc.vmName
 }
