@@ -1,5 +1,8 @@
 // SPDX-License-Identifier: Apache-2.0
 // Copyright Authors of Tetragon
+
+//go:build !windows
+
 package alignchecker
 
 import (
@@ -9,6 +12,7 @@ import (
 	"testing"
 
 	tus "github.com/cilium/tetragon/pkg/testutils/sensors"
+	"github.com/stretchr/testify/require"
 )
 
 var tetragonLib string
@@ -22,12 +26,9 @@ func init() {
 	}
 }
 
-func TestStructAlignments(t *testing.T) {
+func Test_Alignments(t *testing.T) {
 	bpfObjPath := filepath.Join(tetragonLib, "bpf_alignchecker.o")
-	if _, err := os.Stat(bpfObjPath); err != nil {
-		t.Fatalf("Cannot check alignment against %s: %s\n", bpfObjPath, err)
-	}
-	if err := CheckStructAlignments(bpfObjPath); err != nil {
-		t.Fatalf("C and Go structs alignment check failed: %s\n", err)
-	}
+
+	err := CheckStructAlignmentsDefault(bpfObjPath)
+	require.NoError(t, err, "structs must align")
 }
