@@ -1,3 +1,6 @@
+// SPDX-License-Identifier: Apache-2.0
+// Copyright Authors of Tetragon
+
 package dataapi
 
 import "github.com/cilium/tetragon/pkg/api/processapi"
@@ -9,7 +12,9 @@ type DataEventId struct {
 
 type DataEventDesc struct {
 	Error    int32
+	Pad      uint32
 	Leftover uint32
+	Size     uint32
 	Id       DataEventId
 }
 
