@@ -7,6 +7,7 @@ package bugtool
 
 import (
 	"archive/tar"
+	"bufio"
 	"bytes"
 	"compress/gzip"
 	"context"
@@ -14,23 +15,25 @@ import (
 	"errors"
 	"fmt"
 	"io"
+	"log/slog"
+	"net"
 	"net/http"
 	"os"
 	"os/exec"
 	"path"
 	"path/filepath"
+	"strconv"
 	"strings"
 	"time"
 
 	"github.com/cilium/tetragon/api/v1/tetragon"
+	"github.com/cilium/tetragon/cmd/tetra/common"
 	"github.com/cilium/tetragon/pkg/defaults"
 	"github.com/cilium/tetragon/pkg/logger"
 	"github.com/cilium/tetragon/pkg/policyfilter"
+	gopssignal "github.com/google/gops/signal"
 	"go.uber.org/multierr"
-	"google.golang.org/grpc"
-	"google.golang.org/grpc/credentials/insecure"
 
-	"github.com/sirupsen/logrus"
 	"github.com/vishvananda/netlink"
 )
 
@@ -38,12 +41,14 @@ import (
 type InitInfo struct {
 	ExportFname string `json:"export_fname"`
 	LibDir      string `json:"lib_dir"`
-	BtfFname    string `json:"btf_fname"`
+	BTFFname    string `json:"btf_fname"`
 	ServerAddr  string `json:"server_address"`
 	MetricsAddr string `json:"metrics_address"`
 	GopsAddr    string `json:"gops_address"`
 	MapDir      string `json:"map_dir"`
 	BpfToolPath string `json:"bpftool_path"`
+	GopsPath    string `json:"gops_path"`
+	PID         int    `json:"pid"`
 }
 
 // LoadInitInfo returns the InitInfo by reading the info file from its default location
@@ -59,14 +64,14 @@ func SaveInitInfo(info *InitInfo) error {
 func doLoadInitInfo(fname string) (*InitInfo, error) {
 	f, err := os.Open(fname)
 	if err != nil {
-		logger.GetLogger().WithField("infoFile", fname).Warn("failed to open file")
+		logger.GetLogger().Warn("failed to open file", "infoFile", fname)
 		return nil, err
 	}
 	defer f.Close()
 
 	var info InitInfo
 	if err := json.NewDecoder(f).Decode(&info); err != nil {
-		logger.GetLogger().WithField("infoFile", fname).Warn("failed to read information from file")
+		logger.GetLogger().Warn("failed to read information from file", "infoFile", fname)
 		return nil, err
 	}
 
@@ -80,28 +85,36 @@ func doSaveInitInfo(fname string, info *InitInfo) error {
 		logger.GetLogger().Warn("failed to locate bpftool binary, on bugtool debugging ensure you have bpftool installed")
 	} else {
 		info.BpfToolPath = bpftool
-		logger.GetLogger().WithField("bpftool", info.BpfToolPath).Info("Successfully detected bpftool path")
+		logger.GetLogger().Info("Successfully detected bpftool path", "bpftool", info.BpfToolPath)
+	}
+
+	gops, err := exec.LookPath("gops")
+	if err != nil {
+		logger.GetLogger().Warn("failed to locate gops binary, on bugtool debugging ensure you have gops installed")
+	} else {
+		info.GopsPath = gops
+		logger.GetLogger().Info("Successfully detected gops path", "gops", info.GopsPath)
 	}
 
 	// Create DefaultRunDir if it does not already exist
 	if err := os.MkdirAll(defaults.DefaultRunDir, 0755); err != nil {
-		logger.GetLogger().WithField("infoFile", fname).Warn("failed to directory exists")
+		logger.GetLogger().Warn("failed to directory exists", "infoFile", fname)
 		return err
 	}
 	f, err := os.OpenFile(fname, os.O_WRONLY|os.O_CREATE, 0744)
 	if err != nil {
-		logger.GetLogger().WithField("infoFile", fname).Warn("failed to create file")
+		logger.GetLogger().Warn("failed to create file", "infoFile", fname)
 		return err
 	}
 	defer f.Close()
 
 	if err := f.Truncate(0); err != nil {
-		logger.GetLogger().WithField("infoFile", fname).Warn("failed to truncate file")
+		logger.GetLogger().Warn("failed to truncate file", "infoFile", fname)
 		return err
 	}
 
 	if err := json.NewEncoder(f).Encode(info); err != nil {
-		logger.GetLogger().WithField("infoFile", fname).Warn("failed to write information to file")
+		logger.GetLogger().Warn("failed to write information to file", "infoFile", fname)
 		return err
 	}
 
@@ -174,7 +187,7 @@ func (s *bugtoolInfo) tarAddFile(tarWriter *tar.Writer, fnameSrc string, fnameDs
 
 	_, err = io.Copy(tarWriter, fileSrc)
 	if err != nil {
-		s.multiLog.WithField("fnameSrc", fnameSrc).Warn("error copying data from source file")
+		s.multiLog.WithError(err).WithField("fnameSrc", fnameSrc).Warn("error copying data from source file")
 		return err
 	}
 
@@ -182,7 +195,7 @@ func (s *bugtoolInfo) tarAddFile(tarWriter *tar.Writer, fnameSrc string, fnameDs
 }
 
 // Bugtool gathers information and writes it as a tar archive in the given filename
-func Bugtool(outFname string, bpftool string) error {
+func Bugtool(outFname string, bpftool string, gops string) error {
 	info, err := LoadInitInfo()
 	if err != nil {
 		return err
@@ -192,23 +205,27 @@ func Bugtool(outFname string, bpftool string) error {
 		info.BpfToolPath = bpftool
 	}
 
+	if gops != "" {
+		info.GopsPath = gops
+	}
+
 	return doBugtool(info, outFname)
 }
 
 func doBugtool(info *InitInfo, outFname string) error {
 	// we log into two logs, one is the standard one and another one is a
 	// buffer that we are going to include as a file into the bugtool archive.
-	bugtoolLogger := logrus.New()
 	logBuff := new(bytes.Buffer)
-	bugtoolLogger.Out = logBuff
-	logrus.SetLevel(logrus.InfoLevel)
+	bugtoolLogger := slog.New(slog.NewTextHandler(logBuff, &slog.HandlerOptions{
+		Level: slog.LevelInfo,
+	}))
 	multiLog := MultiLog{
-		Logs: []logrus.FieldLogger{
+		Logs: []logger.FieldLogger{
 			logger.GetLogger(),
 			bugtoolLogger,
 		},
 	}
-	prefixDir := fmt.Sprintf("tetragon-bugtool-%s", time.Now().Format("20060102150405"))
+	prefixDir := "tetragon-bugtool-" + time.Now().Format("20060102150405")
 
 	outFile, err := os.Create(outFname)
 	if err != nil {
@@ -234,7 +251,7 @@ func doBugtool(info *InitInfo, outFname string) error {
 
 	si.addInitInfo(tarWriter)
 	si.addLibFiles(tarWriter)
-	si.addBtfFile(tarWriter)
+	si.addBTFFile(tarWriter)
 	si.addTetragonLog(tarWriter)
 	si.addMetrics(tarWriter)
 	si.execCmd(tarWriter, "dmesg.out", "dmesg")
@@ -243,6 +260,10 @@ func doBugtool(info *InitInfo, outFname string) error {
 	si.addGopsInfo(tarWriter)
 	si.dumpPolicyFilterMap(tarWriter)
 	si.addGrpcInfo(tarWriter)
+	si.addPmapOut(tarWriter)
+	si.addMemCgroupStats(tarWriter)
+	si.addBPFMapsStats(tarWriter)
+	si.addTracefsTraceFile(tarWriter)
 	return nil
 }
 
@@ -284,7 +305,7 @@ func (s *bugtoolInfo) addLibFiles(tarWriter *tar.Writer) error {
 			// symlinks if they point within the directory we are archiving, but since
 			// we do not use them, there is currently no reason for the complexity.
 			mode := info.Mode()
-			if !(mode.IsRegular() || mode.IsDir()) {
+			if !mode.IsRegular() && !mode.IsDir() {
 				s.multiLog.WithField("path", path).Warn("not a regular file, ignoring")
 				return nil
 			}
@@ -327,15 +348,15 @@ func (s *bugtoolInfo) addLibFiles(tarWriter *tar.Writer) error {
 		})
 }
 
-// addBtfFile adds the btf file to the archive.
-func (s *bugtoolInfo) addBtfFile(tarWriter *tar.Writer) error {
-	btfFname, err := filepath.EvalSymlinks(s.info.BtfFname)
-	if err != nil && s.info.BtfFname != "" {
-		s.multiLog.WithField("btfFname", s.info.BtfFname).Warnf("error resolving btf file: %s", err)
+// addBTFFile adds the btf file to the archive.
+func (s *bugtoolInfo) addBTFFile(tarWriter *tar.Writer) error {
+	btfFname, err := filepath.EvalSymlinks(s.info.BTFFname)
+	if err != nil && s.info.BTFFname != "" {
+		s.multiLog.WithField("btfFname", s.info.BTFFname).Warnf("error resolving btf file: %s", err)
 		return err
 	}
 
-	if s.info.BtfFname == "" {
+	if s.info.BTFFname == "" {
 		s.multiLog.Warnf("no btf filename in tetragon config, attempting to fall back to /sys/kernel/btf/vmlinux")
 		btfFname = "/sys/kernel/btf/vmlinux"
 	}
@@ -488,7 +509,7 @@ func (s *bugtoolInfo) addBpftoolInfo(tarWriter *tar.Writer) {
 
 	_, err := os.Stat(s.info.BpfToolPath)
 	if err != nil {
-		s.multiLog.WithError(err).Warn("Failed to locate bpftool, please install it.")
+		s.multiLog.WithError(err).Warn("Failed to locate bpftool. Please install it or specify its path, see 'bugtool --help'")
 		return
 	}
 	s.execCmd(tarWriter, "bpftool-maps.json", s.info.BpfToolPath, "map", "show", "-j")
@@ -496,13 +517,76 @@ func (s *bugtoolInfo) addBpftoolInfo(tarWriter *tar.Writer) {
 	s.execCmd(tarWriter, "bpftool-cgroups.json", s.info.BpfToolPath, "cgroup", "tree", "-j")
 }
 
+func (s *bugtoolInfo) getPProf(tarWriter *tar.Writer, file string, gopsSignal byte) error {
+	if s.info.GopsAddr == "" {
+		s.multiLog.Info("Skipping gops dump info as daemon is running without gops, use --gops-address to enable gops")
+		return nil
+	}
+
+	s.multiLog.WithField("gops-address", s.info.GopsAddr).Info("Contacting gops server for pprof dump")
+
+	conn, err := net.Dial("tcp", s.info.GopsAddr)
+	if err != nil {
+		s.multiLog.WithField("gops-address", s.info.GopsAddr).WithError(err).Warn("Failed to contact gops server")
+		return err
+	}
+
+	buf := []byte{gopsSignal}
+	if _, err := conn.Write(buf); err != nil {
+		s.multiLog.WithField("gops-address", s.info.GopsAddr).WithField("file", file).WithError(err).Warn("Failed to send gops pprof command")
+		return err
+	}
+
+	buff := new(bytes.Buffer)
+	if _, err = buff.ReadFrom(conn); err != nil {
+		s.multiLog.WithField("gops-address", s.info.GopsAddr).WithField("file", file).WithError(err).Warn("Failed reading gops pprof response")
+	}
+	return s.tarAddBuff(tarWriter, file, buff)
+}
+
 func (s *bugtoolInfo) addGopsInfo(tarWriter *tar.Writer) {
 	if s.info.GopsAddr == "" {
+		s.multiLog.Info("Skipping gops dump info as daemon is running without gops, use --gops-address to enable gops")
 		return
 	}
-	s.execCmd(tarWriter, "gops.stack", "gops", "stack", s.info.GopsAddr)
-	s.execCmd(tarWriter, "gpos.stats", "gops", "stats", s.info.GopsAddr)
-	s.execCmd(tarWriter, "gops.memstats", "gops", "memstats", s.info.GopsAddr)
+
+	if s.info.GopsPath == "" {
+		s.multiLog.WithField("gops-address", s.info.GopsAddr).Warn("Failed to locate gops. Please install it or specify its path, see 'bugtool --help'")
+		return
+	}
+
+	_, err := os.Stat(s.info.GopsPath)
+	if err != nil {
+		s.multiLog.WithField("gops-address", s.info.GopsAddr).WithError(err).Warn("Failed to locate gops, please install it")
+		return
+	}
+
+	s.multiLog.WithField("gops-address", s.info.GopsAddr).WithField("gops-path", s.info.GopsPath).Info("Dumping gops information")
+
+	s.execCmd(tarWriter, "gops.stack", s.info.GopsPath, "stack", s.info.GopsAddr)
+	s.execCmd(tarWriter, "gops.stats", s.info.GopsPath, "stats", s.info.GopsAddr)
+	s.execCmd(tarWriter, "gops.memstats", s.info.GopsPath, "memstats", s.info.GopsAddr)
+	profiles := map[string]byte{
+		"cpu":  gopssignal.CPUProfile,
+		"heap": gopssignal.HeapProfile,
+	}
+	for name, signal := range profiles {
+		err = s.getPProf(tarWriter, "gops.pprof-"+name, signal)
+		if err != nil {
+			s.multiLog.
+				WithField("gops-address", s.info.GopsAddr).
+				WithField("gops-path", s.info.GopsPath).
+				WithField("profile", name).
+				WithError(err).
+				Warn("Failed to dump gops pprof")
+		} else {
+			s.multiLog.
+				WithField("gops-address", s.info.GopsAddr).
+				WithField("gops-path", s.info.GopsPath).
+				WithField("profile", name).
+				Info("Successfully dumped gops pprof")
+		}
+	}
 }
 
 func (s *bugtoolInfo) dumpPolicyFilterMap(tarWriter *tar.Writer) error {
@@ -522,22 +606,14 @@ func (s *bugtoolInfo) dumpPolicyFilterMap(tarWriter *tar.Writer) error {
 }
 
 func (s *bugtoolInfo) addGrpcInfo(tarWriter *tar.Writer) {
-	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
-	defer cancel()
-	conn, err := grpc.DialContext(
-		ctx,
-		s.info.ServerAddr,
-		grpc.WithTransportCredentials(insecure.NewCredentials()),
-		grpc.WithBlock(),
-	)
+	c, err := common.NewClient(context.Background(), s.info.ServerAddr, 5*time.Second)
 	if err != nil {
-		s.multiLog.Warnf("failed to connect to %s: %v", s.info.ServerAddr, err)
+		s.multiLog.Warnf("failed to create gRPC client to %s: %v", s.info.ServerAddr, err)
 		return
 	}
-	defer conn.Close()
-	client := tetragon.NewFineGuidanceSensorsClient(conn)
+	defer c.Close()
 
-	res, err := client.ListTracingPolicies(ctx, &tetragon.ListTracingPoliciesRequest{})
+	res, err := c.Client.ListTracingPolicies(c.Ctx, &tetragon.ListTracingPoliciesRequest{})
 	if err != nil || res == nil {
 		s.multiLog.Warnf("failed to list tracing policies: %v", err)
 		return
@@ -552,3 +628,172 @@ func (s *bugtoolInfo) addGrpcInfo(tarWriter *tar.Writer) {
 
 	s.multiLog.Infof("dumped tracing policies in %s", fname)
 }
+
+func (s bugtoolInfo) addPmapOut(tarWriter *tar.Writer) error {
+	pmap, err := exec.LookPath("pmap")
+	if err != nil {
+		s.multiLog.WithError(err).Warn("Failed to locate pmap. Please install it.")
+		return fmt.Errorf("failed to locate pmap: %w", err)
+	}
+
+	s.execCmd(tarWriter, "pmap.out", pmap, "-x", strconv.Itoa(s.info.PID))
+	return nil
+}
+
+func findCgroupMountPath(r io.Reader, unified bool, controller string) (string, error) {
+	cgroupName := "cgroup"
+	if unified {
+		cgroupName = "cgroup2"
+	}
+
+	scanner := bufio.NewScanner(r)
+	for scanner.Scan() {
+		line := scanner.Text()
+		fields := strings.Fields(line)
+		if len(fields) >= 3 && (fields[2] == cgroupName) {
+			if unified || !unified && strings.HasSuffix(fields[1], controller) {
+				return fields[1], nil
+			}
+		}
+	}
+
+	if err := scanner.Err(); err != nil {
+		return "", fmt.Errorf("error reading /proc/mounts: %w", err)
+	}
+
+	return "", errors.New("cgroup filesystem not found")
+}
+
+func FindCgroupMountPath(unified bool, controller string) (string, error) {
+	file, err := os.Open("/proc/mounts")
+	if err != nil {
+		return "", fmt.Errorf("failed to open /proc/mounts: %w", err)
+	}
+	defer file.Close()
+	return findCgroupMountPath(file, unified, controller)
+}
+
+func findMemoryCgroupPath(r io.Reader) (bool, string, error) {
+	var unified bool
+	var memoryCgroupPath string
+
+	scanner := bufio.NewScanner(r)
+	for scanner.Scan() {
+		line := scanner.Text()
+
+		// '/proc/$PID/cgroup' lists a process's cgroup membership. If legacy cgroup is
+		// in use in the system, this file may contain multiple lines, one for each
+		// hierarchy. The entry for cgroup v2 is always in the format '0::$PATH'.
+		if strings.HasPrefix(line, "0::/") {
+			unified = true
+			memoryCgroupPath = strings.TrimPrefix(line, "0::")
+
+			// we don't break here because we want to consider cases in which
+			// cgroup v2 line is before other cgroup v1 lines and we want to
+			// consider hybrid as v1, not sure it can happen in real life
+			continue
+		}
+
+		// Parsing for cgroup v1, consider hybrid as v1
+		parts := strings.SplitN(line, ":", 3)
+		if len(parts) == 3 {
+			if parts[1] == "memory" {
+				unified = false
+				memoryCgroupPath = parts[2]
+				break
+			}
+		}
+	}
+
+	if err := scanner.Err(); err != nil {
+		return false, "", fmt.Errorf("failed reading /proc/self/cgroup: %w", err)
+	}
+
+	return unified, memoryCgroupPath, nil
+}
+
+func FindMemoryCgroupPath() (unified bool, memoryCgroupPath string, err error) {
+	file, err := os.Open("/proc/self/cgroup")
+	if err != nil {
+		return false, "", fmt.Errorf("failed to open /proc/self/cgroup: %w", err)
+	}
+	defer file.Close()
+	return findMemoryCgroupPath(file)
+}
+
+func (s bugtoolInfo) addMemCgroupStats(tarWriter *tar.Writer) error {
+	unifiedCgroup, memoryCgroupPath, err := FindMemoryCgroupPath()
+	if err != nil {
+		s.multiLog.WithError(err).Warn("failed finding the memory cgroup path")
+		return fmt.Errorf("failed to find memory cgroup path: %w", err)
+	}
+
+	cgroupMountPath, err := FindCgroupMountPath(unifiedCgroup, "memory")
+	if err != nil {
+		s.multiLog.WithError(err).Warn("failed to find cgroup mount path")
+		return fmt.Errorf("failed to find cgroup mount path: %w", err)
+	}
+
+	cgroupPath := filepath.Join(cgroupMountPath, memoryCgroupPath)
+
+	// can't use s.tarAddFile here unfortunately because it is using io.Copy
+	// based on the size retrieved from the stat of the file, and cgroup fs
+	// files have size equal to 0
+	readAndWrite := func(cgroupBasePath string, file string) error {
+		buf, err := os.ReadFile(filepath.Join(cgroupBasePath, file))
+		if err != nil {
+			s.multiLog.WithError(err).WithField("file", file).Warn("failed to read cgroup file")
+			return fmt.Errorf("failed to read file %s: %w", file, err)
+		}
+		err = s.tarAddBuff(tarWriter, file, bytes.NewBuffer(buf))
+		if err == nil {
+			s.multiLog.WithField("file", file).Info("cgroup file added")
+			return fmt.Errorf("failed to add buffer: %w", err)
+		}
+		return nil
+	}
+
+	if unifiedCgroup {
+		readAndWrite(cgroupPath, "memory.current")
+		readAndWrite(cgroupPath, "memory.stat")
+	} else {
+		err := readAndWrite(cgroupPath, "memory.usage_in_bytes")
+		if err != nil {
+			// Before cgroup namespace, /proc/pid/cgroup mapping was broken, so
+			// Docker back in the days mounted the cgroup hierarchy flat in the
+			// containerfs.  For compatibility, it still does that for cgroup v1.
+			// See more https://lewisgaul.co.uk/blog/coding/2022/05/13/cgroups-intro/#cgroups-and-containers
+			cgroupPath = cgroupMountPath
+			s.multiLog.WithField("cgroupPath", cgroupPath).Info("retrying to read cgroup file from a different legacy path")
+			readAndWrite(cgroupPath, "memory.usage_in_bytes")
+		}
+		readAndWrite(cgroupPath, "memory.kmem.usage_in_bytes")
+		readAndWrite(cgroupPath, "memory.stat")
+	}
+
+	return nil
+}
+
+func (s bugtoolInfo) addBPFMapsStats(tarWriter *tar.Writer) error {
+	out, err := RunMapsChecks(TetragonBPFFS)
+	if err != nil {
+		s.multiLog.WithError(err).Warn("failed to run BPF maps checks")
+		return fmt.Errorf("failed to run BPF maps checks: %w", err)
+	}
+
+	const file = "debugmaps.json"
+	err = s.tarAddJson(tarWriter, file, out)
+	if err != nil {
+		s.multiLog.WithError(err).Warn("failed to add the BPF maps checks to the tar archive")
+		return err
+	}
+	s.multiLog.WithField("file", file).Info("BPF maps checks added")
+	return nil
+}
+
+func (s *bugtoolInfo) addTracefsTraceFile(tarWriter *tar.Writer) {
+	err := s.execCmd(tarWriter, "trace", "cat", "/sys/kernel/tracing/trace")
+	if err != nil {
+		s.multiLog.Warnf("failed to get trace file: %v", err)
+	}
+}
