@@ -9,11 +9,12 @@ import (
 	"fmt"
 	"io"
 	"os"
+	"strings"
 
 	"github.com/cilium/tetragon/api/v1/tetragon"
 	"github.com/cilium/tetragon/pkg/arch"
 	"github.com/cilium/tetragon/pkg/logger"
-	"github.com/cilium/tetragon/pkg/syscallinfo"
+
 	"google.golang.org/protobuf/encoding/protojson"
 )
 
@@ -83,17 +84,21 @@ func (p *TtyEncoder) Encode(v interface{}) error {
 
 // CompactEncoder encodes tetragon.GetEventsResponse in a short format with emojis and colors.
 type CompactEncoder struct {
-	Writer     io.Writer
-	Colorer    *Colorer
-	Timestamps bool
+	Writer      io.Writer
+	Colorer     *Colorer
+	Timestamps  bool
+	StackTraces bool
+	ImaHash     bool
 }
 
 // NewCompactEncoder initializes and returns a pointer to CompactEncoder.
-func NewCompactEncoder(w io.Writer, colorMode ColorMode, timestamps bool) *CompactEncoder {
+func NewCompactEncoder(w io.Writer, colorMode ColorMode, timestamps bool, stackTraces bool, imaHash bool) *CompactEncoder {
 	return &CompactEncoder{
-		Writer:     w,
-		Colorer:    NewColorer(colorMode),
-		Timestamps: timestamps,
+		Writer:      w,
+		Colorer:     NewColorer(colorMode),
+		Timestamps:  timestamps,
+		StackTraces: stackTraces,
+		ImaHash:     imaHash,
 	}
 }
 
@@ -103,7 +108,7 @@ func (p *CompactEncoder) Encode(v interface{}) error {
 	if !ok {
 		return ErrInvalidEvent
 	}
-	logger.GetLogger().WithField("event", v).Debug("Processing event")
+	logger.GetLogger().With("event", v).Debug("Processing event")
 	str, err := p.EventToString(event)
 	if err != nil {
 		return err
@@ -113,6 +118,19 @@ func (p *CompactEncoder) Encode(v interface{}) error {
 		str = fmt.Sprintf("%s %s", ts, str)
 	}
 	fmt.Fprintln(p.Writer, str)
+
+	// print stack trace if available
+	if p.StackTraces {
+		st := HumanStackTrace(event, p.Colorer)
+		fmt.Fprint(p.Writer, st)
+	}
+
+	// print ima hash if available
+	if p.ImaHash {
+		st := HumanIMAHash(event, p.Colorer)
+		fmt.Fprint(p.Writer, st)
+	}
+
 	return nil
 }
 
@@ -152,7 +170,7 @@ const (
 
 func CapTrailorPrinter(str string, caps string) string {
 	if len(caps) == 0 {
-		return fmt.Sprintf("%s", str)
+		return str
 	}
 	padding := 0
 	if len(str) < capsPad {
@@ -188,8 +206,76 @@ func PrintNS(ns int32) string {
 	return nsId[ns]
 }
 
+func HumanStackTrace(response *tetragon.GetEventsResponse, colorer *Colorer) string {
+	out := new(strings.Builder)
+	if ev, ok := response.Event.(*tetragon.GetEventsResponse_ProcessKprobe); ok {
+		if ev.ProcessKprobe.KernelStackTrace != nil {
+			fmt.Fprintf(out, "Kernel:\n")
+			for _, st := range ev.ProcessKprobe.KernelStackTrace {
+				colorer.Green.Fprintf(out, "   0x%x:", st.Address)
+				colorer.Blue.Fprintf(out, " %s", st.Symbol)
+				fmt.Fprintf(out, "+")
+				colorer.Yellow.Fprintf(out, "0x%x\n", st.Offset)
+			}
+		}
+		if ev.ProcessKprobe.UserStackTrace != nil {
+			fmt.Fprintf(out, "User:\n")
+			for _, st := range ev.ProcessKprobe.UserStackTrace {
+				colorer.Green.Fprintf(out, "   0x%x:", st.Address)
+				if st.Symbol != "" {
+					colorer.Blue.Fprintf(out, " %s", st.Symbol)
+				}
+				colorer.Yellow.Fprintf(out, " (%s+0x%x)\n", st.Module, st.Offset)
+			}
+		}
+	}
+	return out.String()
+}
+func HumanIMAHash(response *tetragon.GetEventsResponse, colorer *Colorer) string {
+	out := new(strings.Builder)
+	if ev, ok := response.Event.(*tetragon.GetEventsResponse_ProcessLsm); ok {
+		if ev.ProcessLsm.ImaHash != "" {
+			var path string
+			switch ev.ProcessLsm.FunctionName {
+			case "bprm_check_security":
+				fallthrough
+			case "bprm_committed_creds":
+				fallthrough
+			case "bprm_committing_creds":
+				fallthrough
+			case "bprm_creds_for_exec":
+				fallthrough
+			case "bprm_creds_from_file":
+				path = ev.ProcessLsm.Args[0].GetLinuxBinprmArg().Path
+			case "file_ioctl":
+				fallthrough
+			case "file_lock":
+				fallthrough
+			case "file_open":
+				fallthrough
+			case "file_post_open":
+				fallthrough
+			case "file_receive":
+				fallthrough
+			case "mmap_file":
+				path = ev.ProcessLsm.Args[0].GetFileArg().Path
+			default:
+			}
+			if path != "" {
+				colorer.Green.Fprintf(out, "   %s", path)
+				colorer.Blue.Fprintf(out, " %s\n", ev.ProcessLsm.ImaHash)
+			}
+		}
+	}
+	return out.String()
+}
+
 func (p *CompactEncoder) EventToString(response *tetragon.GetEventsResponse) (string, error) {
 	switch response.Event.(type) {
+	case *tetragon.GetEventsResponse_Test:
+		// This shouldn't normally be reachable since Tetragon won't generate Test
+		// events outside of specific unit tests.
+		return "TEST EVENT", nil
 	case *tetragon.GetEventsResponse_ProcessExec:
 		exec := response.GetProcessExec()
 		if exec.Process == nil {
@@ -214,6 +300,17 @@ func (p *CompactEncoder) EventToString(response *tetragon.GetEventsResponse) (st
 			status = p.Colorer.Red.Sprint(exit.Status)
 		}
 		return CapTrailorPrinter(fmt.Sprintf("%s %s %s %s", event, processInfo, args, status), caps), nil
+	case *tetragon.GetEventsResponse_ProcessThrottle:
+		throttle := response.GetProcessThrottle()
+		event := p.Colorer.Red.Sprintf("🧬 %-7s", "throttle")
+		var typ string
+		switch throttle.Type {
+		case tetragon.ThrottleType_THROTTLE_START:
+			typ = p.Colorer.Red.Sprint("START")
+		case tetragon.ThrottleType_THROTTLE_STOP:
+			typ = p.Colorer.Green.Sprint("STOP ")
+		}
+		return fmt.Sprintf("%s %s %s", event, typ, throttle.Cgroup), nil
 	case *tetragon.GetEventsResponse_ProcessLoader:
 		loader := response.GetProcessLoader()
 		if loader.Process == nil {
@@ -234,7 +331,8 @@ func (p *CompactEncoder) EventToString(response *tetragon.GetEventsResponse) (st
 			return "", ErrMissingProcessInfo
 		}
 		processInfo, caps := p.Colorer.ProcessInfo(response.NodeName, kprobe.Process)
-		switch arch.CutSyscallPrefix(kprobe.FunctionName) {
+		_, sc := arch.CutSyscallPrefix(kprobe.FunctionName)
+		switch sc {
 		case "sys_write":
 			event := p.Colorer.Blue.Sprintf("📝 %-7s", "write")
 			file := ""
@@ -384,12 +482,13 @@ func (p *CompactEncoder) EventToString(response *tetragon.GetEventsResponse) (st
 		case "security_file_permission":
 			event := p.Colorer.Blue.Sprintf("❓ %-7s", "security_file_permission")
 			attr := ""
-			if len(kprobe.Args) > 0 && kprobe.Args[0] != nil && kprobe.Args[1] != nil {
+			if len(kprobe.Args) > 1 && kprobe.Args[0] != nil && kprobe.Args[1] != nil {
 				file := kprobe.Args[0].GetFileArg()
 				action := kprobe.Args[1].GetIntArg()
-				if action == 0x02 {
+				switch action {
+				case 0x02:
 					event = p.Colorer.Blue.Sprintf("📝 %-7s", "write")
-				} else if action == 0x04 {
+				case 0x04:
 					event = p.Colorer.Blue.Sprintf("📚 %-7s", "read")
 				}
 				attr = p.Colorer.Cyan.Sprintf("%s", file.Path)
@@ -398,7 +497,7 @@ func (p *CompactEncoder) EventToString(response *tetragon.GetEventsResponse) (st
 		case "security_mmap_file":
 			event := p.Colorer.Blue.Sprintf("❓ %-7s", "security_mmap_file")
 			attr := ""
-			if len(kprobe.Args) > 0 && kprobe.Args[0] != nil && kprobe.Args[1] != nil {
+			if len(kprobe.Args) > 1 && kprobe.Args[0] != nil && kprobe.Args[1] != nil {
 				file := kprobe.Args[0].GetFileArg()
 				action := kprobe.Args[1].GetUintArg()
 				eventTag := "mmap-"
@@ -443,51 +542,23 @@ func (p *CompactEncoder) EventToString(response *tetragon.GetEventsResponse) (st
 			event := p.Colorer.Blue.Sprintf("⁉️ %-7s", "tracepoint")
 			return CapTrailorPrinter(fmt.Sprintf("%s %s %s %s", event, processInfo, tp.Subsys, tp.Event), caps), nil
 		}
-	}
-
-	return "", ErrUnknownEventType
-}
-
-func rawSyscallEnter(tp *tetragon.ProcessTracepoint) string {
-	sysID := int64(-1)
-	if len(tp.Args) > 0 && tp.Args[0] != nil {
-		if x, ok := tp.Args[0].GetArg().(*tetragon.KprobeArgument_LongArg); ok {
-			sysID = x.LongArg
+	case *tetragon.GetEventsResponse_ProcessUprobe:
+		uprobe := response.GetProcessUprobe()
+		if uprobe.Process == nil {
+			return "", ErrMissingProcessInfo
 		}
-	}
-	sysName := "unknown"
-	if name := syscallinfo.GetSyscallName(int(sysID)); name != "" {
-		sysName = name
-		sysArgs, ok := syscallinfo.GetSyscallArgs(sysName)
-		if ok {
-			sysName += "("
-			for j, arg := range sysArgs {
-				if j > 0 {
-					sysName += ", "
-				}
-				i := j + 1
-
-				argVal := "?"
-				isPtr := false
-				if len(tp.Args) > i && tp.Args[i] != nil {
-					if x, ok := tp.Args[i].GetArg().(*tetragon.KprobeArgument_SizeArg); ok {
-						argVal_ := x.SizeArg
-						if len(arg.Type) > 0 && arg.Type[len(arg.Type)-1] == '*' {
-							isPtr = true
-							argVal = fmt.Sprintf("0x%x", argVal_)
-						} else {
-							argVal = fmt.Sprintf("%d", argVal_)
-						}
-					}
-				}
-				if isPtr {
-					sysName += fmt.Sprintf("%s%s=%s", arg.Type, arg.Name, argVal)
-				} else {
-					sysName += fmt.Sprintf("%s %s=%s", arg.Type, arg.Name, argVal)
-				}
-			}
-			sysName += ")"
+		processInfo, caps := p.Colorer.ProcessInfo(response.NodeName, uprobe.Process)
+		event := p.Colorer.Blue.Sprintf("🕵️ %-7s", "uprobe")
+		return CapTrailorPrinter(fmt.Sprintf("%s %s %s %s", event, processInfo, uprobe.Path, uprobe.Symbol), caps), nil
+	case *tetragon.GetEventsResponse_ProcessLsm:
+		lsm := response.GetProcessLsm()
+		if lsm.Process == nil {
+			return "", ErrMissingProcessInfo
 		}
+		processInfo, caps := p.Colorer.ProcessInfo(response.NodeName, lsm.Process)
+		event := p.Colorer.Blue.Sprintf("🔒 %-7s", "LSM")
+		return CapTrailorPrinter(fmt.Sprintf("%s %s %s", event, processInfo, lsm.FunctionName), caps), nil
 	}
-	return sysName
+
+	return "", fmt.Errorf("%w: %s", ErrUnknownEventType, response.EventType())
 }
