@@ -9,21 +9,25 @@ import (
 	"regexp"
 
 	"github.com/cilium/tetragon/api/v1/tetragon"
-	hubbleV1 "github.com/cilium/tetragon/pkg/oldhubble/api/v1"
-	hubbleFilters "github.com/cilium/tetragon/pkg/oldhubble/filters"
+	"github.com/cilium/tetragon/pkg/event"
 )
 
-func filterByArgumentsRegex(argumentsPatterns []string) (hubbleFilters.FilterFunc, error) {
+func filterByArgumentsRegex(argumentsPatterns []string, parent bool) (FilterFunc, error) {
 	var argsRegexList []*regexp.Regexp
 	for _, pattern := range argumentsPatterns {
 		query, err := regexp.Compile(pattern)
 		if err != nil {
-			return nil, fmt.Errorf("failed to compile regexp: %v", err)
+			return nil, fmt.Errorf("failed to compile regexp: %w", err)
 		}
 		argsRegexList = append(argsRegexList, query)
 	}
-	return func(ev *hubbleV1.Event) bool {
-		process := GetProcess(ev)
+	return func(ev *event.Event) bool {
+		var process *tetragon.Process
+		if parent {
+			process = GetParent(ev)
+		} else {
+			process = GetProcess(ev)
+		}
 		if process == nil {
 			return false
 		}
@@ -38,10 +42,24 @@ func filterByArgumentsRegex(argumentsPatterns []string) (hubbleFilters.FilterFun
 
 type ArgumentsRegexFilter struct{}
 
-func (f *ArgumentsRegexFilter) OnBuildFilter(_ context.Context, ff *tetragon.Filter) ([]hubbleFilters.FilterFunc, error) {
-	var fs []hubbleFilters.FilterFunc
+func (f *ArgumentsRegexFilter) OnBuildFilter(_ context.Context, ff *tetragon.Filter) ([]FilterFunc, error) {
+	var fs []FilterFunc
 	if ff.ArgumentsRegex != nil {
-		argumentsFilters, err := filterByArgumentsRegex(ff.ArgumentsRegex)
+		argumentsFilters, err := filterByArgumentsRegex(ff.ArgumentsRegex, false)
+		if err != nil {
+			return nil, err
+		}
+		fs = append(fs, argumentsFilters)
+	}
+	return fs, nil
+}
+
+type ParentArgumentsRegexFilter struct{}
+
+func (f *ParentArgumentsRegexFilter) OnBuildFilter(_ context.Context, ff *tetragon.Filter) ([]FilterFunc, error) {
+	var fs []FilterFunc
+	if ff.ParentArgumentsRegex != nil {
+		argumentsFilters, err := filterByArgumentsRegex(ff.ParentArgumentsRegex, true)
 		if err != nil {
 			return nil, err
 		}
