@@ -1,13 +1,16 @@
+// SPDX-License-Identifier: Apache-2.0
+// Copyright Authors of Tetragon
+
 package tracing
 
 import (
+	"github.com/cilium/tetragon/pkg/metrics"
 	"github.com/cilium/tetragon/pkg/metrics/consts"
 	"github.com/prometheus/client_golang/prometheus"
-	"github.com/prometheus/client_golang/prometheus/promauto"
 )
 
 var (
-	LoaderStats = promauto.NewCounterVec(prometheus.CounterOpts{
+	LoaderStats = prometheus.NewCounterVec(prometheus.CounterOpts{
 		Namespace:   consts.MetricsNamespace,
 		Name:        "process_loader_stats",
 		Help:        "Process Loader event statistics. For internal use only.",
@@ -15,6 +18,17 @@ var (
 	}, []string{"count"})
 )
 
+func RegisterMetrics(group metrics.Group) {
+	group.MustRegister(LoaderStats)
+}
+
+func InitMetrics() {
+	// Initialize metrics with labels
+	for _, ty := range LoaderTypeStrings {
+		LoaderStats.WithLabelValues(ty).Add(0)
+	}
+}
+
 type LoaderType int
 
 const (
