@@ -1,12 +1,15 @@
 // SPDX-License-Identifier: Apache-2.0
 // Copyright Authors of Tetragon
 
+//go:build !windows
+
 package kernels
 
 import (
 	"testing"
 
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 )
 
 func TestKernelStringToNumeric(t *testing.T) {
@@ -34,10 +37,14 @@ func TestKernelStringToNumeric(t *testing.T) {
 	assert.Less(t, v1, v2)
 	assert.Less(t, v2, v3)
 	assert.Less(t, v1, v3)
+
+	v1 = KernelStringToNumeric("5.4.263")
+	v2 = KernelStringToNumeric("5.5.0")
+	assert.Less(t, v1, v2)
 }
 
 func TestGetKernelVersion(t *testing.T) {
 	ver, verStr, err := GetKernelVersion("", "/proc")
-	assert.Nil(t, err)
+	require.NoError(t, err)
 	assert.EqualValues(t, KernelStringToNumeric(verStr), ver)
 }
