@@ -4,22 +4,23 @@
 package opcodemetrics
 
 import (
-	"fmt"
+	"strconv"
 
+	"github.com/cilium/tetragon/pkg/api/ops"
+	"github.com/cilium/tetragon/pkg/metrics"
 	"github.com/cilium/tetragon/pkg/metrics/consts"
 	"github.com/prometheus/client_golang/prometheus"
-	"github.com/prometheus/client_golang/prometheus/promauto"
 )
 
 var (
-	MsgOpsCount = promauto.NewCounterVec(prometheus.CounterOpts{
+	MsgOpsCount = prometheus.NewCounterVec(prometheus.CounterOpts{
 		Namespace:   consts.MetricsNamespace,
 		Name:        "msg_op_total",
 		Help:        "The total number of times we encounter a given message opcode. For internal use only.",
 		ConstLabels: nil,
 	}, []string{"msg_op"})
 
-	LatencyStats = promauto.NewHistogramVec(prometheus.HistogramOpts{
+	LatencyStats = prometheus.NewHistogramVec(prometheus.HistogramOpts{
 		Namespace:   consts.MetricsNamespace,
 		Name:        "handling_latency",
 		Help:        "The latency of handling messages in us.",
@@ -28,12 +29,27 @@ var (
 	}, []string{"op"})
 )
 
+func RegisterMetrics(group metrics.Group) {
+	group.MustRegister(MsgOpsCount)
+	group.MustRegister(LatencyStats)
+}
+
+func InitMetrics() {
+	// Initialize all metrics
+	for opcode := range ops.OpCodeStrings {
+		if opcode != ops.MSG_OP_UNDEF && opcode != ops.MSG_OP_TEST {
+			GetOpTotal(opcode).Add(0)
+			LatencyStats.WithLabelValues(strconv.Itoa(int(int32(opcode))))
+		}
+	}
+}
+
 // Get a new handle on a msgOpsCount metric for an OpCode
-func GetOpTotal(op int) prometheus.Counter {
-	return MsgOpsCount.WithLabelValues(fmt.Sprint(op))
+func GetOpTotal(opcode ops.OpCode) prometheus.Counter {
+	return MsgOpsCount.WithLabelValues(strconv.Itoa(int(int32(opcode))))
 }
 
 // Increment an msgOpsCount for an OpCode
-func OpTotalInc(op int) {
-	GetOpTotal(op).Inc()
+func OpTotalInc(opcode ops.OpCode) {
+	GetOpTotal(opcode).Inc()
 }
