@@ -1,20 +1,26 @@
+// SPDX-License-Identifier: Apache-2.0
+// Copyright Authors of Tetragon
+
+//go:build !windows
+
 package policyfilter
 
 import (
 	"context"
+	"encoding/hex"
 	"fmt"
+	"log/slog"
 	"math/rand"
+	"sync/atomic"
 	"testing"
 	"time"
 
-	slimv1 "github.com/cilium/cilium/pkg/k8s/slim/k8s/apis/meta/v1"
+	slimv1 "github.com/cilium/tetragon/pkg/k8s/slim/k8s/apis/meta/v1"
 	"github.com/cilium/tetragon/pkg/labels"
 	"github.com/cilium/tetragon/pkg/logger"
 	"github.com/cilium/tetragon/pkg/option"
 	"github.com/google/uuid"
-	"github.com/sirupsen/logrus"
 	"github.com/stretchr/testify/require"
-	"go.uber.org/atomic"
 	v1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	k8stypes "k8s.io/apimachinery/pkg/types"
@@ -41,11 +47,11 @@ import (
 
 type tlog struct {
 	*testing.T
-	Log *logrus.Logger
+	Logger *slog.Logger
 }
 
 func (tl tlog) Write(p []byte) (n int, err error) {
-	tl.Logf((string)(p))
+	tl.Log(string(p))
 	return len(p), nil
 }
 
@@ -66,7 +72,7 @@ type testPod struct {
 func (ts *testState) randString(length int) string {
 	b := make([]byte, length+2)
 	ts.rnd.Read(b)
-	return fmt.Sprintf("%x", b)[2 : length+2]
+	return hex.EncodeToString(b)[2 : length+2]
 }
 
 // testState provides two things: i) helper functions for creating/updating pods, and ii) a fake
@@ -96,7 +102,7 @@ func (ts *testState) callbacksDone() bool {
 
 func (ts *testState) waitForCallbacks(t *testing.T) {
 	dt := 1 * time.Millisecond
-	for i := 0; i < 6; i++ {
+	for range 6 {
 		time.Sleep(dt)
 		if ts.callbacksDone() {
 			return
@@ -111,16 +117,16 @@ func (ts *testState) eventHandler(m *state) cache.ResourceEventHandler {
 	h := m.getPodEventHandlers()
 	return cache.ResourceEventHandlerFuncs{
 		AddFunc: func(obj interface{}) {
+			h.OnAdd(obj, false)
 			ts.cbAdds.Add(1)
-			h.OnAdd(obj)
 		},
 		UpdateFunc: func(oldObj, newObj interface{}) {
-			ts.cbUpds.Add(1)
 			h.OnUpdate(oldObj, newObj)
+			ts.cbUpds.Add(1)
 		},
 		DeleteFunc: func(obj interface{}) {
-			ts.cbDels.Add(1)
 			h.OnDelete(obj)
+			ts.cbDels.Add(1)
 		},
 	}
 }
@@ -299,10 +305,46 @@ func (ts *testState) podsCgroupIDs(t *testing.T, podNames ...string) []uint64 {
 	return ret
 }
 
+// get cgroup IDs of containers from pods
+// podContainerMap is a map of podName -> [podContainerNames]
+func (ts *testState) containersCgroupIDs(t *testing.T, podContainerMap map[string][]string) []uint64 {
+	var ret []uint64
+	for podName, containerNames := range podContainerMap {
+		var p *testPod
+		for _, pod := range ts.pods {
+			if pod.name == podName {
+				p = &pod
+				break
+			}
+		}
+
+		if p == nil {
+			t.Fatalf("unknown pod name: %s", podName)
+		}
+
+		// we create the map to easily understand if the given container name
+		// exists. If not, we raise an exception
+		existingContainerIDNameMap := make(map[string]uint64)
+		for _, cont := range p.containers {
+			existingContainerIDNameMap[cont.name] = uint64(cont.cgID)
+		}
+
+		for i := range containerNames {
+			val, ok := existingContainerIDNameMap[containerNames[i]]
+			if !ok {
+				t.Fatalf("unknown container name: %s", containerNames[i])
+			}
+			ret = append(ret, val)
+		}
+	}
+
+	return ret
+}
+
 func testNamespacePods(t *testing.T, st *state, ts *testState) {
-	err := st.AddPolicy(PolicyID(1), "ns1", nil)
+	err := st.AddPolicy(PolicyID(1), "ns1", nil, nil)
 	require.NoError(t, err)
-	err = st.AddPolicy(PolicyID(2), "ns2", nil)
+	err = st.AddPolicy(PolicyID(2), "ns2", nil, nil)
 	require.NoError(t, err)
 
 	emptyLabels := labels.Labels{}
@@ -363,7 +405,7 @@ func testPodLabelFilters(t *testing.T, st *state, ts *testState) {
 	matchesAllID := uint32(1)
 	matchesWebID := uint32(2)
 	matchesAppsID := uint32(3)
-	err := st.AddPolicy(PolicyID(matchesAllID), "", nil)
+	err := st.AddPolicy(PolicyID(matchesAllID), "", nil, nil)
 	require.NoError(t, err)
 	err = st.AddPolicy(PolicyID(matchesWebID), "", &slimv1.LabelSelector{
 		MatchExpressions: []slimv1.LabelSelectorRequirement{{
@@ -371,14 +413,14 @@ func testPodLabelFilters(t *testing.T, st *state, ts *testState) {
 			Operator: slimv1.LabelSelectorOpIn,
 			Values:   []string{"web"},
 		}},
-	})
+	}, nil)
 	require.NoError(t, err)
 	err = st.AddPolicy(PolicyID(matchesAppsID), "", &slimv1.LabelSelector{
 		MatchExpressions: []slimv1.LabelSelectorRequirement{{
 			Key:      "app",
 			Operator: slimv1.LabelSelectorOpExists,
 		}},
-	})
+	}, nil)
 	require.NoError(t, err)
 
 	// create pods
@@ -448,6 +490,134 @@ func testPodLabelFilters(t *testing.T, st *state, ts *testState) {
 	)
 }
 
+func testContainerFieldFilters(t *testing.T, st *state, ts *testState) {
+	// create policies
+	matchesAllContainers := uint32(1)
+	matchesWebContainers := uint32(2)
+	matchesNotInitContainers := uint32(3)
+	err := st.AddPolicy(PolicyID(matchesAllContainers), "", nil, nil)
+	require.NoError(t, err)
+	err = st.AddPolicy(PolicyID(matchesWebContainers), "", nil,
+		&slimv1.LabelSelector{
+			MatchExpressions: []slimv1.LabelSelectorRequirement{{
+				Key:      "name",
+				Operator: slimv1.LabelSelectorOpIn,
+				Values:   []string{"web-c1", "web-c2", "web-c3"},
+			}},
+		})
+	require.NoError(t, err)
+	err = st.AddPolicy(PolicyID(matchesNotInitContainers), "", &slimv1.LabelSelector{
+		MatchExpressions: []slimv1.LabelSelectorRequirement{{
+			Key:      "app",
+			Operator: slimv1.LabelSelectorOpIn,
+			Values:   []string{"web", "db", "log"},
+		}},
+	}, &slimv1.LabelSelector{
+		MatchExpressions: []slimv1.LabelSelectorRequirement{{
+			Key:      "name",
+			Operator: slimv1.LabelSelectorOpNotIn,
+			Values:   []string{"init"},
+		}},
+	})
+	require.NoError(t, err)
+
+	// create pods
+	ts.createPod(t, "web", "default", labels.Labels{"app": "web"}, "web-c1", "web-c2", "init")
+	ts.createPod(t, "db", "default", labels.Labels{"app": "db"}, "db-c1")
+	ts.createPod(t, "log", "default", labels.Labels{}, "log-c1", "init")
+
+	ts.waitForCallbacks(t)
+	requirePfmEqualTo(t, st.pfMap,
+		map[uint64][]uint64{
+			uint64(matchesAllContainers): ts.containersCgroupIDs(t, map[string][]string{
+				"web": {"web-c1", "web-c2", "init"},
+				"db":  {"db-c1"},
+				"log": {"log-c1", "init"},
+			}),
+			uint64(matchesWebContainers): ts.containersCgroupIDs(t, map[string][]string{
+				"web": {"web-c1", "web-c2"},
+			}),
+			// test policy state before we label the log with the matching label
+			uint64(matchesNotInitContainers): ts.containersCgroupIDs(t, map[string][]string{
+				"web": {"web-c1", "web-c2"},
+				"db":  {"db-c1"},
+				"log": {},
+			}),
+		},
+	)
+
+	// make sure adding this label will not add all cgroup IDs from the pod to the policy
+	ts.updatePodLabels(t, "log", labels.Labels{"app": "log"})
+	ts.updatePodContainers(t, "web", "web-c3", "app-c1")
+	ts.waitForCallbacks(t)
+	requirePfmEqualTo(t, st.pfMap,
+		map[uint64][]uint64{
+			uint64(matchesAllContainers): ts.containersCgroupIDs(t, map[string][]string{
+				"web": {"web-c3", "app-c1"},
+				"db":  {"db-c1"},
+				"log": {"log-c1", "init"},
+			}),
+			uint64(matchesWebContainers): ts.containersCgroupIDs(t, map[string][]string{
+				"web": {"web-c3"},
+			}),
+			uint64(matchesNotInitContainers): ts.containersCgroupIDs(t, map[string][]string{
+				"web": {"web-c3", "app-c1"},
+				"db":  {"db-c1"},
+				"log": {"log-c1"},
+			}),
+		},
+	)
+
+	// make sure removing this label will not try to remove all cgroup IDs of this pod from the policy
+	// it will not raise an error but there will be a warning
+	ts.updatePodLabels(t, "log", labels.Labels{"app": "not-log"})
+	ts.updatePodContainers(t, "db", "db-c1", "init")
+	ts.waitForCallbacks(t)
+	requirePfmEqualTo(t, st.pfMap,
+		map[uint64][]uint64{
+			uint64(matchesAllContainers): ts.containersCgroupIDs(t, map[string][]string{
+				"web": {"web-c3", "app-c1"},
+				"db":  {"db-c1", "init"},
+				"log": {"log-c1", "init"},
+			}),
+			uint64(matchesWebContainers): ts.containersCgroupIDs(t, map[string][]string{
+				"web": {"web-c3"},
+			}),
+			uint64(matchesNotInitContainers): ts.containersCgroupIDs(t, map[string][]string{
+				"web": {"web-c3", "app-c1"},
+				"db":  {"db-c1"},
+			}),
+		},
+	)
+
+	err = st.DelPolicy(PolicyID(matchesAllContainers))
+	require.NoError(t, err)
+	ts.deletePod(t, "log")
+	ts.waitForCallbacks(t)
+	requirePfmEqualTo(t, st.pfMap,
+		map[uint64][]uint64{
+			uint64(matchesWebContainers): ts.containersCgroupIDs(t, map[string][]string{
+				"web": {"web-c3"},
+			}),
+			uint64(matchesNotInitContainers): ts.containersCgroupIDs(t, map[string][]string{
+				"web": {"web-c3", "app-c1"},
+				"db":  {"db-c1"},
+			}),
+		},
+	)
+
+	ts.deletePod(t, "web")
+	ts.deletePod(t, "db")
+	err = st.DelPolicy(PolicyID(matchesNotInitContainers))
+	require.NoError(t, err)
+	err = st.DelPolicy(PolicyID(matchesWebContainers))
+	require.NoError(t, err)
+	ts.waitForCallbacks(t)
+	requirePfmEqualTo(t, st.pfMap,
+		map[uint64][]uint64{},
+	)
+}
+
 func testPreExistingPods(t *testing.T, st *state, ts *testState) {
 	// create pods
 	ts.createPod(t, "web", "default", labels.Labels{"app": "web"}, "web-c1", "web-c2")
@@ -462,9 +632,10 @@ func testPreExistingPods(t *testing.T, st *state, ts *testState) {
 			Operator: slimv1.LabelSelectorOpIn,
 			Values:   []string{"web"},
 		}},
-	})
+	}, nil)
 	require.NoError(t, err)
 
+	require.Len(t, ts.podsCgroupIDs(t, "web"), 2)
 	requirePfmEqualTo(t, st.pfMap,
 		map[uint64][]uint64{
 			uint64(matchesWebID): ts.podsCgroupIDs(t, "web"),
@@ -484,15 +655,25 @@ func testPreExistingPods(t *testing.T, st *state, ts *testState) {
 func testContainersChange(t *testing.T, st *state, ts *testState) {
 	ts.createPod(t, "web", "default", nil, "web-c1", "web-c2")
 	ts.createPod(t, "db", "default", nil, "db-c1")
+	ts.createPod(t, "log", "default", nil, "log-c1")
 	ts.waitForCallbacks(t)
 
 	// create policy
 	policyID := uint32(2)
-	err := st.AddPolicy(PolicyID(policyID), "", &slimv1.LabelSelector{})
+	err := st.AddPolicy(PolicyID(policyID), "", &slimv1.LabelSelector{},
+		&slimv1.LabelSelector{
+			MatchExpressions: []slimv1.LabelSelectorRequirement{
+				{
+					Key:      "name",
+					Operator: slimv1.LabelSelectorOpNotIn,
+					Values:   []string{"log-c1"},
+				}},
+		})
 	require.NoError(t, err)
 
-	require.Equal(t, len(ts.podsCgroupIDs(t, "web")), 2)
-	require.Equal(t, len(ts.podsCgroupIDs(t, "db")), 1)
+	require.Len(t, ts.podsCgroupIDs(t, "web"), 2)
+	require.Len(t, ts.podsCgroupIDs(t, "db"), 1)
+	require.Empty(t, ts.containersCgroupIDs(t, map[string][]string{"log": {}}))
 	requirePfmEqualTo(t, st.pfMap,
 		map[uint64][]uint64{
 			uint64(policyID): ts.podsCgroupIDs(t, "web", "db"),
@@ -501,8 +682,8 @@ func testContainersChange(t *testing.T, st *state, ts *testState) {
 
 	ts.updatePodContainers(t, "web", "web-c1", "web-c3", "web-c4")
 	ts.updatePodContainers(t, "db", "db-c2")
-	require.Equal(t, len(ts.podsCgroupIDs(t, "web")), 3)
-	require.Equal(t, len(ts.podsCgroupIDs(t, "db")), 1)
+	require.Len(t, ts.podsCgroupIDs(t, "web"), 3)
+	require.Len(t, ts.podsCgroupIDs(t, "db"), 1)
 	ts.waitForCallbacks(t)
 	requirePfmEqualTo(t, st.pfMap,
 		map[uint64][]uint64{
@@ -512,6 +693,7 @@ func testContainersChange(t *testing.T, st *state, ts *testState) {
 
 	ts.deletePod(t, "web")
 	ts.deletePod(t, "db")
+	ts.deletePod(t, "log")
 	err = st.DelPolicy(PolicyID(policyID))
 	require.NoError(t, err)
 	ts.waitForCallbacks(t)
@@ -526,9 +708,9 @@ func TestK8s(t *testing.T) {
 	defer cancel()
 
 	// NB: using testutils.CaptureLog causes import cycle
-	log := logger.GetLogger().(*logrus.Logger)
-	lc := &tlog{T: t, Log: log}
-	log.SetOutput(lc)
+	lc := &tlog{T: t}
+	log := slog.New(slog.NewTextHandler(lc, nil))
+	logger.DefaultSlogLogger = log
 
 	oldEnablePolicyFilterValue := option.Config.EnablePolicyFilter
 	oldEnablePolicyFilterValueDebug := option.Config.EnablePolicyFilterDebug
@@ -560,9 +742,9 @@ func TestK8s(t *testing.T) {
 
 	// testState implements cgFinder
 	ts := newTestState(client)
-	st, err := newState(log, ts)
+	st, err := newState(log, ts, true)
 	if err != nil {
-		t.Skip(fmt.Sprintf("failed to initialize policy filter state: %s", err))
+		t.Skipf("failed to initialize policy filter state: %s", err)
 	}
 	defer st.Close()
 
@@ -594,6 +776,10 @@ func TestK8s(t *testing.T) {
 		testPodLabelFilters(t, st, ts)
 	})
 
+	t.Run("container fields", func(t *testing.T) {
+		testContainerFieldFilters(t, st, ts)
+	})
+
 	t.Run("pre-existing pods", func(t *testing.T) {
 		testPreExistingPods(t, st, ts)
 	})
