@@ -1,3 +1,8 @@
+// SPDX-License-Identifier: Apache-2.0
+// Copyright Authors of Tetragon
+
+//go:build !windows
+
 package policyfilter
 
 import (
@@ -16,9 +21,20 @@ func requirePfmEqualTo(t *testing.T, m PfMap, val map[uint64][]uint64) {
 		}
 	}
 
+	checkCgroupVals := map[CgroupID]map[PolicyID]struct{}{}
+	for k, ids := range val {
+		for _, id := range ids {
+			if checkCgroupVals[CgroupID(id)] == nil {
+				checkCgroupVals[CgroupID(id)] = map[PolicyID]struct{}{}
+			}
+			checkCgroupVals[CgroupID(id)][PolicyID(k)] = struct{}{}
+		}
+	}
+
 	mapVals, err := m.readAll()
 	require.NoError(t, err)
-	require.EqualValues(t, checkVals, mapVals)
+	require.Equal(t, checkVals, mapVals.Policy)
+	require.Equal(t, checkCgroupVals, mapVals.Cgroup)
 }
 
 // TestPfMapOps tests some simple map operations
@@ -26,7 +42,7 @@ func TestPfMapOps(t *testing.T) {
 	if !bpffsReady {
 		t.Skip("failed to initialize bpffs")
 	}
-	pfm, err := newPfMap()
+	pfm, err := newPfMap(true)
 	require.NoError(t, err)
 	defer pfm.release()
 
@@ -39,9 +55,11 @@ func TestPfMapOps(t *testing.T) {
 
 	err = pm1.addCgroupIDs([]CgroupID{30})
 	require.NoError(t, err)
+	err = addPolicyIDMapping(pm1.cgroupMap, polID1, 30)
+	require.NoError(t, err)
 	requirePfmEqualTo(t, pfm, map[uint64][]uint64{100: {10, 20, 30}})
 
-	err = pm1.delCgroupIDs([]CgroupID{20, 10})
+	err = pm1.delCgroupIDs(polID1, []CgroupID{20, 10})
 	require.NoError(t, err)
 	requirePfmEqualTo(t, pfm, map[uint64][]uint64{100: {30}})
 
