@@ -5,17 +5,21 @@ package policyfilter
 
 import (
 	"fmt"
+	"log/slog"
 	"sync"
 
-	slimv1 "github.com/cilium/cilium/pkg/k8s/slim/k8s/apis/meta/v1"
+	"github.com/cilium/ebpf"
 	"github.com/cilium/tetragon/pkg/cgroups/fsscan"
+	slimv1 "github.com/cilium/tetragon/pkg/k8s/slim/k8s/apis/meta/v1"
 	"github.com/cilium/tetragon/pkg/labels"
 	"github.com/cilium/tetragon/pkg/logger"
-	pfmetrics "github.com/cilium/tetragon/pkg/metrics/policyfilter"
+	"github.com/cilium/tetragon/pkg/logger/logfields"
+	"github.com/cilium/tetragon/pkg/metrics/policyfiltermetrics"
+	"github.com/cilium/tetragon/pkg/option"
+	"github.com/cilium/tetragon/pkg/podhelpers"
 	"github.com/cilium/tetragon/pkg/podhooks"
 
 	"github.com/google/uuid"
-	"github.com/sirupsen/logrus"
 	v1 "k8s.io/api/core/v1"
 	"k8s.io/client-go/tools/cache"
 )
@@ -27,6 +31,7 @@ import (
 // The pods that match a given policy are selected based on:
 //   (1) Namespaces
 //   (2) Label filters
+//   (3) Container field filters
 //
 // This package maintains the 'policy_filter_maps' bpf map. Bpf checks this map
 // to decide whether a policy is applied or not. The map is a hash-of-hashes:
@@ -40,13 +45,13 @@ import (
 // needs to be updated in the following conditions:
 //
 //  (A) Policy changes: when a new policy is added (or deleted), we need to add cgroup ids of
-//  matching pods. See {Add,Del}Policy.
+//  matching containers inside pods. See {Add,Del}Policy.
 //
 //  (B) Pod containers changes: when new containers are added (or deleted): we need to add the cgroup
-//  ids of matching policies. See AddPodContainer, DelPodContainer, DelPod.
+//  ids of matching policies. See AddPodContainer, DelPodContainer, DelPod, UpdatePod.
 //
 //  (C) Pod labels change: need to rescan policies because the result of pod label filters might have
-//  changed.
+//  changed. See UpdatePod.
 //
 // Todo:
 //  - use a goroutine and a queue
@@ -70,11 +75,15 @@ const (
 	// should be large enough to accommodate the number of containers
 	// running in a system.
 	polMapSize = 32768
+
+	// same as POLICY_FILTER_MAX_POLICIES in policy_filter.h
+	polMaxPolicies = 128
 )
 
 type PolicyID uint32
 type PodID uuid.UUID
 type CgroupID uint64
+type StateID uint64
 
 const (
 	// we reserve 0 as a special value to indicate no filtering
@@ -84,13 +93,15 @@ const (
 )
 
 func (i PodID) String() string {
-	var x uuid.UUID = uuid.UUID(i)
+	var x = uuid.UUID(i)
 	return x.String()
 }
 
 type containerInfo struct {
 	id   string   // container id
 	cgID CgroupID // cgroup id
+	name string   // container name
+	repo string   // container repo
 }
 
 // podInfo contains the necessary information for each pod
@@ -98,20 +109,14 @@ type podInfo struct {
 	id         PodID
 	namespace  string
 	labels     labels.Labels
+	workload   string
+	kind       string
 	containers []containerInfo
 
 	// cache of matched policies
 	matchedPolicies []PolicyID
 }
 
-func (pod *podInfo) cgroupIDs() []CgroupID {
-	ret := make([]CgroupID, 0, len(pod.containers))
-	for i := range pod.containers {
-		ret = append(ret, pod.containers[i].cgID)
-	}
-	return ret
-}
-
 // delete containers from a pod based on their id, and return them
 // NB: in most cases there will be a single container, but we do not reject users adding a container
 // with the same id and different cgroup, so we return a list to cover all cases.
@@ -137,7 +142,7 @@ func (pod *podInfo) delCachedPolicy(polID PolicyID) {
 }
 
 func (pod *podInfo) addCachedPolicy(polID PolicyID) {
-	for i := 0; i < len(pod.matchedPolicies); i++ {
+	for i := range len(pod.matchedPolicies) {
 		if pod.matchedPolicies[i] == polID {
 			return
 		}
@@ -146,7 +151,7 @@ func (pod *podInfo) addCachedPolicy(polID PolicyID) {
 }
 
 func (pod *podInfo) hasPolicy(polID PolicyID) bool {
-	for i := 0; i < len(pod.matchedPolicies); i++ {
+	for i := range len(pod.matchedPolicies) {
 		if pod.matchedPolicies[i] == polID {
 			return true
 		}
@@ -177,12 +182,11 @@ func (m *state) containerExists(pod *podInfo, containerID string, cgIDp *CgroupI
 		// NB: if this happens, we might end up with multiple cgroup
 		// ids for the same container. Since we have no way of knowing
 		// which one is the correct, we keep both.
-		m.log.WithFields(logrus.Fields{
-			"pod-id":        pod.id,
-			"container-id":  containerID,
-			"old-cgroup-id": container.cgID,
-			"new-cgroup-id": *cgIDp,
-		}).Warnf("AddPodContainer: conflicting cgroup ids")
+		m.log.Warn("AddPodContainer: conflicting cgroup ids",
+			"pod-id", pod.id,
+			"container-id", containerID,
+			"old-cgroup-id", container.cgID,
+			"new-cgroup-id", *cgIDp)
 	}
 
 	return false
@@ -194,6 +198,8 @@ type policy struct {
 	// if namespace is "", policy applies to all namespaces
 	namespace string
 
+	containerSelector labels.Selector
+
 	podSelector labels.Selector
 
 	// polMap is the (inner) policy map for this policy
@@ -204,17 +210,48 @@ func (pol *policy) podMatches(podNs string, podLabels labels.Labels) bool {
 	if pol.namespace != "" && podNs != pol.namespace {
 		return false
 	}
+	var podLabels1 labels.Labels
+	if podLabels != nil {
+		podLabels1 = podLabels
+	} else {
+		podLabels1 = make(labels.Labels)
+	}
 
-	return pol.podSelector.Match(podLabels)
+	if _, ok := podLabels1[labels.K8sPodNamespace]; !ok {
+		podLabels1[labels.K8sPodNamespace] = podNs
+	}
+
+	return pol.podSelector.Match(podLabels1)
 }
 
 func (pol *policy) podInfoMatches(pod *podInfo) bool {
 	return pol.podMatches(pod.namespace, pod.labels)
 }
 
+func (pol *policy) containerMatches(container *containerInfo) bool {
+	containerFilterFields := labels.Labels{
+		"name": container.name,
+		"repo": container.repo,
+	}
+	return pol.containerSelector.Match(containerFilterFields)
+}
+
+// get a slice of container cgroupIDs that match the policy
+func (pol *policy) matchingContainersCgroupIDs(containers []containerInfo) []CgroupID {
+	var cgroupIDs []CgroupID
+	for i := range containers {
+		container := containers[i]
+		if pol.containerMatches(&container) {
+			cgroupIDs = append(cgroupIDs, container.cgID)
+		}
+	}
+	return cgroupIDs
+}
+
 // State holds the necessary state for policyfilter
 type state struct {
-	log logrus.FieldLogger
+	log *slog.Logger
+	*logger.DebugLogger
 
 	// mutex serializes access to the internal structures, as well as operations.
 	mu       sync.Mutex
@@ -224,6 +261,9 @@ type state struct {
 	// polify filters (outer) map handle
 	pfMap PfMap
 
+	// global policy map handle
+	nsMap *NamespaceMap
+
 	cgidFinder cgidFinder
 }
 
@@ -231,25 +271,33 @@ type state struct {
 // allocated resources (namely the bpf map).
 //
 //revive:disable:unexported-return
-func New() (*state, error) {
-	log := logger.GetLogger().WithField("subsystem", "policy-filter")
+func New(enableCgroupMap bool) (*state, error) {
+	log := logger.GetLogger().With("subsystem", "policy-filter")
 	return newState(
 		log,
 		&cgfsFinder{fsscan.New(), log},
+		enableCgroupMap,
 	)
 }
 
 func newState(
-	log logrus.FieldLogger,
+	log *slog.Logger,
 	cgidFinder cgidFinder,
+	enableCgroupMap bool,
 ) (*state, error) {
 	var err error
 	ret := &state{
-		log:        log,
-		cgidFinder: cgidFinder,
+		log:         log,
+		cgidFinder:  cgidFinder,
+		DebugLogger: logger.NewDebugLogger(log, option.Config.EnablePolicyFilterDebug),
 	}
 
-	ret.pfMap, err = newPfMap()
+	ret.pfMap, err = newPfMap(enableCgroupMap)
+	if err != nil {
+		return nil, err
+	}
+
+	ret.nsMap, err = newNamespaceMap()
 	if err != nil {
 		return nil, err
 	}
@@ -260,21 +308,28 @@ func newState(
 //revive:enable:unexported-return
 
 func (m *state) updatePodHandler(pod *v1.Pod) error {
-	containerIDs := podContainersIDs(pod)
+	containerIDs := podhelpers.PodContainersIDs(pod)
+	containerInfo := podhelpers.PodContainersInfo(pod)
 	podID, err := uuid.Parse(string(pod.UID))
 	if err != nil {
-		m.log.WithError(err).WithField("pod-id", pod.UID).Warn("policyfilter, pod handler: failed to parse pod id")
+		m.log.Warn("policyfilter, pod handler: failed to parse pod id", logfields.Error, err, "pod-id", pod.UID)
 		return err
 	}
 
 	namespace := pod.Namespace
-	err = m.UpdatePod(PodID(podID), namespace, pod.Labels, containerIDs)
+	workloadMeta, kindMeta := podhelpers.GetWorkloadMetaFromPod(pod)
+	workload := workloadMeta.Name
+	kind := kindMeta.Kind
+
+	err = m.UpdatePod(PodID(podID), namespace, workload, kind, pod.Labels, containerIDs, containerInfo)
 	if err != nil {
-		m.log.WithError(err).WithFields(logrus.Fields{
-			"pod-id":        podID,
-			"container-ids": containerIDs,
-			"namespace":     namespace,
-		}).Warn("policyfilter, add pod-handler: AddPodContainer failed")
+		m.log.Warn("policyfilter, UpdatePod failed",
+			logfields.Error, err,
+			"pod-id", podID,
+			"container-ids", containerIDs,
+			"namespace", namespace,
+			"workload", workload,
+			"kind", kind)
 		return err
 	}
 
@@ -286,43 +341,43 @@ func (m *state) getPodEventHandlers() cache.ResourceEventHandlerFuncs {
 		AddFunc: func(obj interface{}) {
 			pod, ok := obj.(*v1.Pod)
 			if !ok {
-				logger.GetLogger().Warn("policyfilter, add-pod handler: unexpected object type: %T", pod)
+				logger.GetLogger().Warn(fmt.Sprintf("policyfilter, add-pod handler: unexpected object type: %T", pod))
 				return
 			}
 			err := m.updatePodHandler(pod)
-			pfmetrics.OpInc("pod-handlers", "add", err)
+			policyfiltermetrics.OpInc(policyfiltermetrics.PodHandlersSubsys, policyfiltermetrics.AddPodOperation, ErrorLabel(err))
 		},
-		UpdateFunc: func(oldObj, newObj interface{}) {
+		UpdateFunc: func(_, newObj interface{}) {
 			pod, ok := newObj.(*v1.Pod)
 			if !ok {
-				logger.GetLogger().Warn("policyfilter, update-pod: unexpected object type(s): new:%T", pod)
+				logger.GetLogger().Warn(fmt.Sprintf("policyfilter, update-pod handler: unexpected object type(s): new:%T", pod))
 				return
 			}
 			err := m.updatePodHandler(pod)
-			pfmetrics.OpInc("pod-handlers", "update", err)
+			policyfiltermetrics.OpInc(policyfiltermetrics.PodHandlersSubsys, policyfiltermetrics.UpdatePodOperation, ErrorLabel(err))
 		},
 		DeleteFunc: func(obj interface{}) {
 			// Remove all containers for this pod
 			pod, ok := obj.(*v1.Pod)
 			if !ok {
-				logger.GetLogger().Warn("policyfilter, delete-pod handler: unexpected object type: %T", pod)
+				logger.GetLogger().Warn(fmt.Sprintf("policyfilter, delete-pod handler: unexpected object type: %T", pod))
 				return
 			}
 			podID, err := uuid.Parse(string(pod.UID))
 			if err != nil {
-				logger.GetLogger().WithField("pod-id", pod.UID).WithError(err).Warn("policyfilter, delete-pod: failed to parse id")
+				logger.GetLogger().Warn("policyfilter, delete-pod: failed to parse id", "pod-id", pod.UID, logfields.Error, err)
 				return
 			}
 
 			namespace := pod.Namespace
 			err = m.DelPod(PodID(podID))
 			if err != nil {
-				logger.GetLogger().WithError(err).WithFields(logrus.Fields{
-					"pod-id":    podID,
-					"namespace": namespace,
-				}).Warn("policyfilter, delete-pod handler: DelPod failed")
+				logger.GetLogger().Warn("policyfilter, delete-pod handler: DelPod failed",
+					logfields.Error, err,
+					"pod-id", podID,
+					"namespace", namespace)
 			}
-			pfmetrics.OpInc("pod-handlers", "delete", err)
+			policyfiltermetrics.OpInc(policyfiltermetrics.PodHandlersSubsys, policyfiltermetrics.DeletePodOperation, ErrorLabel(err))
 		},
 	}
 }
@@ -377,7 +432,8 @@ func (m *state) delPod(id PodID) *podInfo {
 }
 
 // AddPolicy adds a policy
-func (m *state) AddPolicy(polID PolicyID, namespace string, podLabelSelector *slimv1.LabelSelector) error {
+func (m *state) AddPolicy(polID PolicyID, namespace string, podLabelSelector *slimv1.LabelSelector,
+	containerLabelSelector *slimv1.LabelSelector) error {
 	m.mu.Lock()
 	defer m.mu.Unlock()
 
@@ -389,10 +445,17 @@ func (m *state) AddPolicy(polID PolicyID, namespace string, podLabelSelector *sl
 	if err != nil {
 		return err
 	}
+
+	containerSelector, err := labels.SelectorFromLabelSelector(containerLabelSelector)
+	if err != nil {
+		return err
+	}
+
 	policy := policy{
-		id:          polID,
-		namespace:   namespace,
-		podSelector: podSelector,
+		id:                polID,
+		namespace:         namespace,
+		podSelector:       podSelector,
+		containerSelector: containerSelector,
 	}
 
 	cgroupIDs := make([]CgroupID, 0)
@@ -403,9 +466,10 @@ func (m *state) AddPolicy(polID PolicyID, namespace string, podLabelSelector *sl
 		if !policy.podInfoMatches(pod) {
 			continue
 		}
-		for cIdx := range pod.containers {
-			cgroupIDs = append(cgroupIDs, pod.containers[cIdx].cgID)
-		}
+
+		matchedContainerCgroupIDs := policy.matchingContainersCgroupIDs(pod.containers)
+		cgroupIDs = append(cgroupIDs, matchedContainerCgroupIDs...)
+
 		matchedPods = append(matchedPods, pod)
 		pod.addCachedPolicy(policy.id)
 	}
@@ -424,7 +488,7 @@ func (m *state) AddPolicy(polID PolicyID, namespace string, podLabelSelector *sl
 	return nil
 }
 
-// DelPolicy will destroly all information for the provided policy
+// DelPolicy will destroy all information for the provided policy
 func (m *state) DelPolicy(polID PolicyID) error {
 
 	if polID == NoFilterPolicyID {
@@ -435,15 +499,18 @@ func (m *state) DelPolicy(polID PolicyID) error {
 	defer m.mu.Unlock()
 	policy := m.delPolicy(polID)
 	if policy != nil {
-		policy.polMap.Close()
+		policy.polMap.Inner.Close()
 	} else {
-		m.log.WithField("policy-id", polID).Warn("DelPolicy: policy internal map not found")
+		m.log.Warn("DelPolicy: policy internal map not found", "policy-id", polID)
 	}
 
-	if err := m.pfMap.Delete(polID); err != nil {
-		m.log.WithField("policy-id", polID).Warn("DelPolicy: failed to remove policy from external map")
+	if err := m.pfMap.policyMap.Delete(polID); err != nil {
+		m.log.Warn("DelPolicy: failed to remove policy from external map", "policy-id", polID)
 	}
 
+	// update cgroup map
+	m.pfMap.deletePolicyIDInCgroupMap(polID)
+
 	for i := range m.pods {
 		pod := &m.pods[i]
 		pod.delCachedPolicy(policy.id)
@@ -459,29 +526,76 @@ func cgIDPointerStr(p *CgroupID) string {
 	return fmt.Sprintf("%d", *p)
 }
 
+// addCgroupIDs add cgroups ids to the policy map
+// todo: use batch operations when supported
+func (m *state) addCgroupIDs(cinfo []containerInfo, pod *podInfo) error {
+	nsmap := m.nsMap
+
+	for _, c := range cinfo {
+		key := NSID{
+			Namespace: pod.namespace,
+			Workload:  pod.workload,
+			Kind:      pod.kind,
+		}
+		id, ok := nsmap.nsNameMap.Get(key)
+		if ok {
+			if err := nsmap.cgroupIdMap.Update(&c.cgID, id, ebpf.UpdateAny); err != nil {
+				logger.GetLogger().Warn("Unable to assign cgroup to existing namespace", logfields.Error, err)
+			}
+			continue
+		}
+		logger.GetLogger().Debug("update cgroupid map", "cgrp", c, "pod", pod, "id", nsmap.id)
+
+		// If this is a new namespace we create a new map entry and bind it to a stable id.
+		if err := nsmap.cgroupIdMap.Update(&c.cgID, nsmap.id, ebpf.UpdateAny); err != nil {
+			logger.GetLogger().Warn("Unable to insert cgroup id map",
+				logfields.Error, err,
+				"cgid", c.cgID,
+				"id", nsmap.id,
+				"ns", c.name)
+			continue
+		}
+		if ok := nsmap.nsIdMap.Add(nsmap.id, key); ok {
+			logger.GetLogger().Info("Id to namespace map caused eviction",
+				"cgid", c.cgID,
+				"id", nsmap.id,
+				"ns", c.name)
+		}
+		if ok := nsmap.nsNameMap.Add(key, nsmap.id); ok {
+			logger.GetLogger().Info("Namespace to Id map caused eviction",
+				"cgid", c.cgID,
+				"id", nsmap.id,
+				"ns", c.name)
+		}
+		nsmap.id++
+	}
+
+	return nil
+}
+
 // addPodContainers adds a list of containers (ids) to a pod.
 // It will update the state for all containers that do not exist.
 // It takes an optional argument of a list of cgroup ids (one per container). If this list is empty,
 // the function will try to figure out the cgroup id on its own.
 // Finally, it will scan over all the matching policies for the pod and update the policy maps.
-func (m *state) addPodContainers(pod *podInfo, containerIDs []string, cgroupIDs []CgroupID) {
+func (m *state) addPodContainers(pod *podInfo, containerIDs []string,
+	cgroupIDs []CgroupID, containerData []podhelpers.ContainerInfo) {
 	// Find the containers that do not exist in our state, and for those find the cgroup id if
 	// one does not exist.
 	cinfo := make([]containerInfo, 0, len(containerIDs))
-	cgIDs := make([]CgroupID, 0, len(containerIDs))
 	for i, contID := range containerIDs {
 		var cgIDptr *CgroupID
 		if len(cgroupIDs) > i {
 			cgIDptr = &cgroupIDs[i]
 		}
+		containerData := containerData[i]
 
 		if m.containerExists(pod, contID, cgIDptr) {
-			m.debugLogWithCallers(4).WithFields(logrus.Fields{
-				"pod-id":       pod.id,
-				"namespace":    pod.namespace,
-				"container-id": contID,
-				"cgroup-id":    cgIDPointerStr(cgIDptr),
-			}).Info("addPodContainers: container exists, skipping")
+			m.DebugLogWithCallers(4).Info("addPodContainers: container exists, skipping",
+				"pod-id", pod.id,
+				"namespace", pod.namespace,
+				"container-id", contID,
+				"cgroup-id", cgIDPointerStr(cgIDptr))
 			continue
 		}
 
@@ -489,62 +603,73 @@ func (m *state) addPodContainers(pod *podInfo, containerIDs []string, cgroupIDs
 			cgid, err := m.cgidFinder.findCgroupID(pod.id, contID)
 			if err != nil {
 				// error: skip this container id
-				m.log.WithError(err).WithFields(logrus.Fields{
-					"pod-id":       pod.id,
-					"container-id": contID,
-				}).Warn("failed to find cgroup id. Skipping container.")
+				m.log.Warn("failed to find cgroup id. Skipping container.",
+					logfields.Error, err,
+					"pod-id", pod.id,
+					"container-id", contID)
 				continue
 			}
 			cgIDptr = &cgid
 		}
 
-		cinfo = append(cinfo, containerInfo{contID, *cgIDptr})
-		cgIDs = append(cgIDs, *cgIDptr)
+		cinfo = append(cinfo, containerInfo{contID, *cgIDptr, containerData.Name, containerData.Repo})
 	}
 
 	if len(cinfo) == 0 {
-		m.debugLogWithCallers(4).WithFields(logrus.Fields{
-			"pod-id":        pod.id,
-			"namespace":     pod.namespace,
-			"container-ids": containerIDs,
-		}).Info("addPodContainers: nothing to do, returning")
+		m.DebugLogWithCallers(4).Info("addPodContainers: nothing to do, returning",
+			"pod-id", pod.id,
+			"namespace", pod.namespace,
+			"container-ids", containerIDs)
 		return
 	}
 
 	// update containers
 	pod.containers = append(pod.containers, cinfo...)
-	m.debugLogWithCallers(4).WithFields(logrus.Fields{
-		"pod-id":          pod.id,
-		"namespace":       pod.namespace,
-		"containers-info": cinfo,
-	}).Info("addPodContainers: container(s) added")
+	m.DebugLogWithCallers(4).Info("addPodContainers: container(s) added",
+		"pod-id", pod.id,
+		"namespace", pod.namespace,
+		"containers-info", cinfo)
+
+	m.addCgroupIDs(cinfo, pod)
 
 	// update matching policy maps
 	for _, policyID := range pod.matchedPolicies {
 		pol := m.findPolicy(policyID)
 		if pol == nil {
-			m.log.WithFields(logrus.Fields{
-				"policy-id":  policyID,
-				"pod-id":     pod.id,
-				"cgroup-ids": cgroupIDs,
-			}).Warn("addPodContainers: unknown policy id found in pod. This should not happen, ignoring.")
+			m.log.Warn("addPodContainers: unknown policy id found in pod. This should not happen, ignoring.",
+				"policy-id", policyID,
+				"pod-id", pod.id,
+				"cgroup-ids", cgroupIDs)
 			continue
 		}
 
-		if err := pol.polMap.addCgroupIDs(cgIDs); err != nil {
-			m.log.WithError(err).WithFields(logrus.Fields{
-				"policy-id":  pol.id,
-				"pod-id":     pod.id,
-				"cgroup-ids": cgroupIDs,
-			}).Warn("failed to update policy map")
+		// cgroup IDs of containers that match the policy
+		matchingCgIDs := pol.matchingContainersCgroupIDs(pod.containers)
+
+		if err := pol.polMap.addCgroupIDs(matchingCgIDs); err != nil {
+			m.log.Warn("failed to update policy map",
+				logfields.Error, err,
+				"policy-id", pol.id,
+				"pod-id", pod.id,
+				"cgroup-ids", matchingCgIDs)
+		} else {
+			if err := pol.polMap.addPolicyIDs(pol.id, matchingCgIDs); err != nil {
+				m.log.Warn("failed to update cgroup map",
+					logfields.Error, err,
+					"policy-id", pol.id,
+					"pod-id", pod.id,
+					"cgroup-ids", matchingCgIDs)
+			}
 		}
 	}
 }
 
-func (m *state) addNewPod(podID PodID, namespace string, podLabels labels.Labels) *podInfo {
+func (m *state) addNewPod(podID PodID, namespace, workload, kind string, podLabels labels.Labels) *podInfo {
 	m.pods = append(m.pods, podInfo{
 		id:         podID,
 		namespace:  namespace,
+		workload:   workload,
+		kind:       kind,
 		labels:     podLabels,
 		containers: nil,
 	})
@@ -562,25 +687,27 @@ func (m *state) addNewPod(podID PodID, namespace string, podLabels labels.Labels
 // if the cgroup id of the container is known, cgID is not nil and it contains its value.
 //
 // The pod might or might not have been encountered before.
-func (m *state) AddPodContainer(podID PodID, namespace string, podLabels labels.Labels, containerID string, cgID CgroupID) error {
+func (m *state) AddPodContainer(podID PodID, namespace, workload, kind string, podLabels labels.Labels,
+	containerID string, cgID CgroupID, containerInfo podhelpers.ContainerInfo) error {
 	m.mu.Lock()
 	defer m.mu.Unlock()
 
 	pod := m.findPod(podID)
 	if pod == nil {
-		pod = m.addNewPod(podID, namespace, podLabels)
-		m.debugLogWithCallers(4).WithFields(logrus.Fields{
-			"pod-id":       podID,
-			"namespace":    namespace,
-			"container-id": containerID,
-			"cgroup-id":    cgID,
-		}).Info("AddPodContainer: added pod")
+		pod = m.addNewPod(podID, namespace, workload, kind, podLabels)
+		m.DebugLogWithCallers(4).Info("AddPodContainer: added pod",
+			"pod-id", podID,
+			"namespace", namespace,
+			"workload", workload,
+			"container-id", containerID,
+			"cgroup-id", cgID,
+			"container-info", containerInfo)
 	} else if pod.namespace != namespace {
 		// sanity check: old and new namespace should match
-		return fmt.Errorf("conflicting namespaces for pod with id %s: old='%s' vs new='%s'", podID, pod.namespace, namespace)
+		return &podNamespaceConflictError{podID: podID, oldNs: pod.namespace, newNs: namespace}
 	}
 
-	m.addPodContainers(pod, []string{containerID}, []CgroupID{cgID})
+	m.addPodContainers(pod, []string{containerID}, []CgroupID{cgID}, []podhelpers.ContainerInfo{containerInfo})
 	return nil
 }
 
@@ -592,30 +719,26 @@ func (m *state) delPodCgroupIDsFromPolicyMaps(pod *podInfo, containers []contain
 		return
 	}
 
-	cgroupIDs := make([]CgroupID, 0, len(containers))
-	for i := range containers {
-		cgroupIDs = append(cgroupIDs, containers[i].cgID)
-	}
-
 	// check what policies match the pod, and delete the cgroup ids
 	for _, policyID := range pod.matchedPolicies {
 		pol := m.findPolicy(policyID)
 		if pol == nil {
-			m.log.WithFields(logrus.Fields{
-				"policy-id":  policyID,
-				"pod-id":     pod.id,
-				"cgroup-ids": cgroupIDs,
-			}).Warn("delPodCgroupIDsFromPolicyMaps: unknown policy id found in pod. This should not happen, ignoring.")
+			m.log.Warn("delPodCgroupIDsFromPolicyMaps: unknown policy id found in pod. This should not happen, ignoring.",
+				"policy-id", policyID,
+				"pod-id", pod.id)
 			continue
 		}
 
-		if err := pol.polMap.delCgroupIDs(cgroupIDs); err != nil {
+		// try to find containers in the pod matching this policy
+		// this way, we only remove containers that are actually present in the policy
+		cgroupIDs := pol.matchingContainersCgroupIDs(containers)
+		if err := pol.polMap.delCgroupIDs(pol.id, cgroupIDs); err != nil {
 			// NB: depending on the error, we might want to schedule some retries here
-			m.log.WithError(err).WithFields(logrus.Fields{
-				"policy-id":  pol.id,
-				"pod-id":     pod.id,
-				"cgroup-ids": cgroupIDs,
-			}).Warn("delPodCgroupIDsFromPolicyMaps: failed to delete cgroup ids from policy map")
+			m.log.Warn("delPodCgroupIDsFromPolicyMaps: failed to delete cgroup ids from policy map",
+				logfields.Error, err,
+				"policy-id", pol.id,
+				"pod-id", pod.id,
+				"cgroup-ids", cgroupIDs)
 		}
 	}
 }
@@ -666,6 +789,7 @@ type policiesDiffRes struct {
 func (m *state) policiesDiff(pod *podInfo, newLabels labels.Labels) *policiesDiffRes {
 	addedPolicies := []*policy{}
 	deletedPolicies := []*policy{}
+
 	newMatchedPolicies := []PolicyID{}
 	for i := range m.policies {
 		pol := &m.policies[i]
@@ -701,26 +825,38 @@ func (m *state) applyPodPolicyDiff(pod *podInfo, polDiff *policiesDiffRes) {
 		return
 	}
 
-	cgroupIDs := pod.cgroupIDs()
+	var cgroupIDs []CgroupID
 	for _, addPol := range polDiff.addedPolicies {
+		cgroupIDs = addPol.matchingContainersCgroupIDs(pod.containers)
 		if err := addPol.polMap.addCgroupIDs(cgroupIDs); err != nil {
-			m.log.WithError(err).WithFields(logrus.Fields{
-				"policy-id":  addPol.id,
-				"pod-id":     pod.id,
-				"cgroup-ids": cgroupIDs,
-				"reason":     "labels change caused policy to match",
-			}).Warn("failed to update policy map")
+			m.log.Warn("failed to update policy map",
+				logfields.Error, err,
+				"policy-id", addPol.id,
+				"pod-id", pod.id,
+				"cgroup-ids", cgroupIDs,
+				"reason", "labels change caused policy to match")
+		} else {
+			// update cgroup map if addCgroupIDs succeeds
+			if err := addPol.polMap.addPolicyIDs(addPol.id, cgroupIDs); err != nil {
+				m.log.Warn("failed to update cgroup map",
+					logfields.Error, err,
+					"policy-id", addPol.id,
+					"pod-id", pod.id,
+					"cgroup-ids", cgroupIDs,
+					"reason", "labels change caused policy to match")
+			}
 		}
 	}
 
 	for _, delPol := range polDiff.deletedPolicies {
-		if err := delPol.polMap.delCgroupIDs(cgroupIDs); err != nil {
-			m.log.WithError(err).WithFields(logrus.Fields{
-				"policy-id":  delPol.id,
-				"pod-id":     pod.id,
-				"cgroup-ids": cgroupIDs,
-				"reason":     "labels change caused policy to unmatch",
-			}).Warn("failed to update policy map")
+		cgroupIDs = delPol.matchingContainersCgroupIDs(pod.containers)
+		if err := delPol.polMap.delCgroupIDs(delPol.id, cgroupIDs); err != nil {
+			m.log.Warn("failed to update policy map",
+				logfields.Error, err,
+				"policy-id", delPol.id,
+				"pod-id", pod.id,
+				"cgroup-ids", cgroupIDs,
+				"reason", "labels change caused policy to unmatch")
 		}
 	}
 	pod.matchedPolicies = polDiff.newMatchedPolicies
@@ -758,27 +894,30 @@ func (pod *podInfo) containerDiff(newContainerIDs []string) ([]string, []string)
 // UpdatePod updates the pod state for a pod
 // containerIDs contains all the running container ids for the given pod.
 // This function will:
+//   - check if pod labels changed. Add/delete policies if this is the case
 //   - remove the containers that are not part of the containerIDs list
 //   - add the ones that do not exist in the current state
 //
 // It is intended to be used from k8s watchers (where no cgroup information is available)
-func (m *state) UpdatePod(podID PodID, namespace string, podLabels labels.Labels, containerIDs []string) error {
+func (m *state) UpdatePod(podID PodID, namespace, workload, kind string, podLabels labels.Labels,
+	containerIDs []string, containerInfo []podhelpers.ContainerInfo) error {
 	m.mu.Lock()
 	defer m.mu.Unlock()
 
-	dlog := m.debugLogWithCallers(4).WithFields(logrus.Fields{
-		"pod-id":        podID,
-		"namespace":     namespace,
-		"container-ids": containerIDs,
-	})
+	dlog := m.DebugLogWithCallers(4).With(
+		"pod-id", podID,
+		"namespace", namespace,
+		"container-ids", containerIDs,
+		"container-info", containerInfo,
+	)
 
 	pod := m.findPod(podID)
 	if pod == nil {
-		pod = m.addNewPod(podID, namespace, podLabels)
+		pod = m.addNewPod(podID, namespace, workload, kind, podLabels)
 		dlog.Info("UpdatePod: added pod")
 	} else if pod.namespace != namespace {
 		// sanity check: old and new namespace should match
-		return fmt.Errorf("conflicting namespaces for pod with id %s: old='%s' vs new='%s'", podID, pod.namespace, namespace)
+		return &podNamespaceConflictError{podID: podID, oldNs: pod.namespace, newNs: namespace}
 	}
 
 	// labels changed: check if there are policies ads that:
@@ -787,12 +926,11 @@ func (m *state) UpdatePod(podID PodID, namespace string, podLabels labels.Labels
 	// and update state accordingly
 	if pod.labels.Cmp(podLabels) {
 		polDiff := m.policiesDiff(pod, podLabels)
-		m.debugLogWithCallers(1).WithFields(logrus.Fields{
-			"pod-id":         pod.id,
-			"pod-old-labels": pod.labels,
-			"pod-new-labels": podLabels,
-			"policy-diff":    fmt.Sprintf("%+v", polDiff),
-		}).Info("UpdatePod: pod labels changed")
+		m.DebugLogWithCallers(1).Info("UpdatePod: pod labels changed",
+			"pod-id", pod.id,
+			"pod-old-labels", pod.labels,
+			"pod-new-labels", podLabels,
+			"policy-diff", fmt.Sprintf("%+v", polDiff))
 		m.applyPodPolicyDiff(pod, polDiff)
 		pod.labels = podLabels
 	}
@@ -808,6 +946,32 @@ func (m *state) UpdatePod(podID PodID, namespace string, podLabels labels.Labels
 		m.delPodCgroupIDsFromPolicyMaps(pod, containers)
 	}
 
-	m.addPodContainers(pod, addIDs, nil)
+	// container names that are associated with addIDs
+	var addContainerInfo []podhelpers.ContainerInfo
+
+	// find container names of the container IDs that should be added to pod
+	for addID := range addIDs {
+		for allID := range containerIDs {
+			if addIDs[addID] == containerIDs[allID] {
+				addContainerInfo = append(addContainerInfo, containerInfo[allID])
+			}
+		}
+	}
+
+	m.addPodContainers(pod, addIDs, nil, addContainerInfo)
 	return nil
 }
+
+func (m *state) GetNsId(stateID StateID) (*NSID, bool) {
+	if ns, ok := m.nsMap.nsIdMap.Get(stateID); ok {
+		return &ns, ok
+	}
+	return nil, false
+}
+
+func (m *state) GetIdNs(id NSID) (StateID, bool) {
+	if stateID, ok := m.nsMap.nsNameMap.Get(id); ok {
+		return stateID, ok
+	}
+	return StateID(0), false
+}
