@@ -1,5 +1,6 @@
 // SPDX-License-Identifier: Apache-2.0
 // Copyright Authors of Tetragon
+
 package process
 
 import (
@@ -10,10 +11,7 @@ import (
 )
 
 func argsDecoderTrim(r rune) bool {
-	if r == 0x00 {
-		return true
-	}
-	return false
+	return r == 0x00
 }
 
 func ArgsDecoder(s string, flags uint32) (string, string) {
