@@ -7,6 +7,7 @@ import (
 	"testing"
 
 	"github.com/cilium/tetragon/api/v1/tetragon"
+	"github.com/cilium/tetragon/pkg/defaults"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 	"google.golang.org/protobuf/types/known/wrapperspb"
@@ -14,7 +15,7 @@ import (
 
 func TestProcessCache(t *testing.T) {
 	// add a process to the cache.
-	cache, err := NewCache(10)
+	cache, err := NewCache(10, defaults.DefaultProcessCacheGCInterval)
 	require.NoError(t, err)
 	pid := wrapperspb.UInt32Value{Value: 1234}
 	execID := "process1"
@@ -31,16 +32,16 @@ func TestProcessCache(t *testing.T) {
 		},
 	}
 	cache.add(&proc)
-	assert.Equal(t, cache.len(), 1)
+	assert.Equal(t, 1, cache.len())
 
 	result, err := cache.get(proc.process.ExecId)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	assert.Equal(t, proc.process.ExecId, result.process.ExecId)
 	assert.Equal(t, proc.capabilities, result.capabilities)
 
 	// remove the entry from cache.
 	assert.True(t, cache.remove(proc.process))
-	assert.Equal(t, cache.len(), 0)
+	assert.Equal(t, 0, cache.len())
 	_, err = cache.get(proc.process.ExecId)
-	assert.Error(t, err)
+	require.Error(t, err)
 }
