@@ -1,9 +1,9 @@
+// SPDX-License-Identifier: Apache-2.0
+// Copyright Authors of Tetragon
+
 package notify
 
 import (
-	"fmt"
-	"strings"
-
 	"github.com/cilium/tetragon/api/v1/tetragon"
 	"github.com/cilium/tetragon/pkg/process"
 )
@@ -19,16 +19,20 @@ type Message interface {
 type Event interface {
 	GetProcess() *tetragon.Process
 	GetParent() *tetragon.Process
+	GetAncestors() []*tetragon.Process
 	SetProcess(*tetragon.Process)
 	SetParent(*tetragon.Process)
+	SetAncestors([]*tetragon.Process)
 	Encapsulate() tetragon.IsGetEventsResponse_Event
 }
 
-func EventTypeString(event Event) string {
-	// Get the concrete type of the event
-	ty := fmt.Sprintf("%T", event)
-	// Take only what comes after the last "."
-	tys := strings.Split(ty, ".")
-	ty = tys[len(tys)-1]
-	return ty
+func EventType(event Event) tetragon.EventType {
+	if event == nil {
+		return tetragon.EventType_UNDEF
+	}
+	eventWrapper := event.Encapsulate()
+	res := &tetragon.GetEventsResponse{
+		Event: eventWrapper,
+	}
+	return res.EventType()
 }
