@@ -14,10 +14,10 @@ import (
 
 type Runner struct {
 	callbacks []Callbacks
-	watcher   watcher.K8sResourceWatcher
+	watcher   watcher.PodAccessor
 }
 
-// runHooks executes all registered callbacks
+// RunHooks executes all registered callbacks
 func (r *Runner) RunHooks(ctx context.Context, req *v1.RuntimeHookRequest) error {
 	if createReq := req.GetCreateContainer(); createReq != nil {
 		var ret error
@@ -42,7 +42,7 @@ func (r *Runner) registerCallbacks(cbs Callbacks) {
 }
 
 // WithWatcher sets the watcher on a runner
-func (r *Runner) WithWatcher(watcher watcher.K8sResourceWatcher) *Runner {
+func (r *Runner) WithWatcher(watcher watcher.PodAccessor) *Runner {
 	r.watcher = watcher
 	return r
 }
