@@ -1,10 +1,14 @@
 // SPDX-License-Identifier: Apache-2.0
 // Copyright Authors of Tetragon
+
+//go:build !windows
+
 package exec
 
 import (
 	"bytes"
 	"context"
+	"errors"
 	"fmt"
 	"os"
 	"os/exec"
@@ -14,20 +18,16 @@ import (
 	"testing"
 	"time"
 
-	"github.com/cilium/tetragon/pkg/kernels"
-
 	"github.com/cilium/tetragon/pkg/api/ops"
 	"github.com/cilium/tetragon/pkg/api/processapi"
 	"github.com/cilium/tetragon/pkg/cgroups"
 	grpcexec "github.com/cilium/tetragon/pkg/grpc/exec"
+	"github.com/cilium/tetragon/pkg/kernels"
 	"github.com/cilium/tetragon/pkg/mountinfo"
-	"github.com/cilium/tetragon/pkg/sensors"
 
 	"github.com/cilium/tetragon/pkg/bpf"
 	"github.com/cilium/tetragon/pkg/logger"
-	"github.com/cilium/tetragon/pkg/observer"
 	"github.com/cilium/tetragon/pkg/option"
-	"github.com/cilium/tetragon/pkg/sensors/base"
 	"github.com/cilium/tetragon/pkg/sensors/cgroup/cgrouptrackmap"
 	"github.com/cilium/tetragon/pkg/sensors/config/confmap"
 	"github.com/cilium/tetragon/pkg/sensors/exec/procevents"
@@ -35,7 +35,6 @@ import (
 	"github.com/cilium/tetragon/pkg/testutils"
 	"github.com/cilium/tetragon/pkg/testutils/perfring"
 	tus "github.com/cilium/tetragon/pkg/testutils/sensors"
-	"github.com/sirupsen/logrus"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 )
@@ -67,14 +66,11 @@ const (
 	tetragonCgrpRoot = "tetragon-tests"
 
 	invalidValue = ^uint32(0)
+
+	tetragonTraceLevel = uint32(6) // Trace level LogLevel_LOG_LEVEL_TRACE for Tetragon
 )
 
 var (
-	loadedSensors = []*sensors.Sensor{
-		testsensor.GetTestSensor(),
-		testsensor.GetCgroupSensor(),
-	}
-
 	defaultKubeCgroupHierarchy = []cgroupHierarchy{
 		{"tetragon-tests-39d631b6f0fbc4e261c7a0ee636cf434-defaultKubeCgroupHierarchy-system.slice", true, false, false, false, 0, 0, "", "", nil},
 		{"kubelet.slice", true, false, false, false, 0, 0, "", "", nil},
@@ -126,7 +122,7 @@ func logTetragonConfig(t *testing.T, mapDir string) error {
 	}
 
 	t.Logf("Test %s tetragon configuration: cgroup.magic=%s  logLevel=%d  cgroup.hierarchyID=%d  cgroup.subsysIdx=%d  cgroup.trackinglevel=%d  cgroup.ID=%d",
-		t.Name(), cgroups.CgroupFsMagicStr(conf.CgrpFsMagic), conf.LogLevel, conf.TgCgrpHierarchy, conf.TgCgrpSubsysIdx, conf.TgCgrpLevel, conf.TgCgrpId)
+		t.Name(), cgroups.CgroupFsMagicStr(conf.CgrpFsMagic), conf.LogLevel, conf.TgCgrpHierarchy, conf.TgCgrpv1SubsysIdx, conf.TgCgrpLevel, conf.TgCgrpId)
 
 	return nil
 }
@@ -204,7 +200,7 @@ func mountCgroupv1Controllers(t *testing.T, cgroupRoot string, usedController st
 	for i, controller := range controllers {
 		hierarchy := filepath.Join(cgroupRoot, controller.name)
 		err := os.MkdirAll(hierarchy, 0555)
-		assert.NoError(t, err)
+		require.NoError(t, err)
 		_, err = mountCgroup(t, hierarchy, "cgroup", controller.name)
 		if err != nil {
 			t.Logf("mountCgroup() %s failed: %v", hierarchy, err)
@@ -221,7 +217,7 @@ func mountCgroupv1Controllers(t *testing.T, cgroupRoot string, usedController st
 	}
 
 	if mountedControllers == 0 {
-		return nil, fmt.Errorf("failed to mount cgroupv1 controllers")
+		return nil, errors.New("failed to mount cgroupv1 controllers")
 	}
 	if usedControllerMounted == false {
 		// cleanup now
@@ -302,17 +298,16 @@ func getCgroupEventOpAndPath(t *testing.T, msg *grpcexec.MsgCgroupEventUnix, cgr
 
 	op := ops.CgroupOpCode(msg.CgrpOp)
 	st := ops.CgroupState(msg.CgrpData.State).String()
-	logger.GetLogger().WithFields(logrus.Fields{
-		"cgroup.event":       op.String(),
-		"PID":                msg.PID,
-		"NSPID":              msg.NSPID,
-		"cgroup.IDTracker":   msg.CgrpidTracker,
-		"cgroup.ID":          msg.Cgrpid,
-		"cgroup.state":       st,
-		"cgroup.hierarchyID": msg.CgrpData.HierarchyId,
-		"cgroup.level":       msg.CgrpData.Level,
-		"cgroup.path":        cgrpPath,
-	}).Info("Received Cgroup event")
+	logger.GetLogger().Info("Received Cgroup event",
+		"cgroup.event", op.String(),
+		"PID", msg.PID,
+		"NSPID", msg.NSPID,
+		"cgroup.IDTracker", msg.CgrpidTracker,
+		"cgroup.ID", msg.Cgrpid,
+		"cgroup.state", st,
+		"cgroup.hierarchyID", msg.CgrpData.HierarchyId,
+		"cgroup.level", msg.CgrpData.Level,
+		"cgroup.path", cgrpPath)
 
 	// match only our target cgroup paths
 	if strings.HasPrefix(cgrpPath, filepath.Join("/", cgroupHierarchy[0].path)) == false {
@@ -322,7 +317,7 @@ func getCgroupEventOpAndPath(t *testing.T, msg *grpcexec.MsgCgroupEventUnix, cgr
 	require.NotZero(t, msg.PID)
 	require.NotZero(t, msg.Cgrpid)
 	require.NotZero(t, msg.CgrpidTracker)
-	require.NotEqualValues(t, msg.CgrpData.State, ops.CGROUP_UNTRACKED)
+	require.NotEqualValues(t, ops.CGROUP_UNTRACKED, msg.CgrpData.State)
 	require.NotZero(t, msg.CgrpData.Level)
 
 	return op, cgrpPath
@@ -332,17 +327,17 @@ func requireCgroupEventOpMkdir(t *testing.T, msg *grpcexec.MsgCgroupEventUnix, c
 	require.EqualValues(t, ops.CGROUP_NEW, msg.CgrpData.State)
 
 	looked, err := cgrouptrackmap.LookupTrackingCgroup(cgrpMapPath, msg.CgrpidTracker)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	if looked == nil {
 		t.Fatalf("Failed to find tracking cgroupID=%d in bpf-map=%s", msg.CgrpidTracker, cgrpMapPath)
 	}
-	require.EqualValues(t, msg.CgrpData.Level, looked.Level)
-	require.EqualValues(t, msg.CgrpData.Name, looked.Name)
+	require.Equal(t, msg.CgrpData.Level, looked.Level)
+	require.Equal(t, msg.CgrpData.Name, looked.Name)
 }
 
 func requireCgroupEventOpRmdir(t *testing.T, msg *grpcexec.MsgCgroupEventUnix, cgrpMapPath string) {
 	looked, err := cgrouptrackmap.LookupTrackingCgroup(cgrpMapPath, msg.CgrpidTracker)
-	assert.Error(t, err)
+	require.Error(t, err)
 	if looked != nil {
 		t.Fatalf("Failed found tracking cgroup with cgroupID=%d in bpf-map=%s", msg.CgrpidTracker, cgrpMapPath)
 	}
@@ -351,16 +346,16 @@ func requireCgroupEventOpRmdir(t *testing.T, msg *grpcexec.MsgCgroupEventUnix, c
 func assertCgroupDirTracking(t *testing.T, cgroupHierarchy []cgroupHierarchy, assertRmdir bool) {
 	for i, c := range cgroupHierarchy {
 		if c.tracking == true {
-			assert.Equalf(t, true, c.added,
+			assert.Truef(t, c.added,
 				"failed at cgroupHierarchy[%d].path=%s should be tracked and added into bpf-map", i, c.path)
 			if assertRmdir {
-				assert.Equalf(t, true, c.removed,
+				assert.Truef(t, c.removed,
 					"failed at cgroupHierarchy[%d].path=%s should be tracked and removed from bpf-map", i, c.path)
 			}
 		} else {
-			assert.Equalf(t, false, c.added,
+			assert.Falsef(t, c.added,
 				"failed at cgroupHierarchy[%d].path=%s should not be tracked nor added into bpf-map", i, c.path)
-			assert.Equalf(t, false, c.removed,
+			assert.Falsef(t, c.removed,
 				"failed at cgroupHierarchy[%d].path=%s should not be tracked nor added/removed from bpf-map", i, c.path)
 		}
 	}
@@ -418,38 +413,38 @@ func assertCgroupv1Events(ctx context.Context, t *testing.T, selectedController
 				switch op {
 				case ops.MSG_OP_CGROUP_MKDIR:
 					require.EqualValues(t, ops.CGROUP_NEW, msg.CgrpData.State)
-					require.EqualValues(t, cgroupHierarchiesMap[controller][msg.CgrpData.Level-1].path, cgrpName)
+					require.Equal(t, cgroupHierarchiesMap[controller][msg.CgrpData.Level-1].path, cgrpName)
 
 					// Get cgroup id from path
 					targetPath := filepath.Join(cgroups.GetCgroupFSPath(), controller, cgrpPath)
 					id, err := cgroups.GetCgroupIdFromPath(targetPath)
 					require.NoErrorf(t, err, "failed to get cgroup ID from path %s", targetPath)
 					// Assert that received cgroup id from event is same as the id from the cgroup fs
-					require.EqualValues(t, msg.CgrpidTracker, id)
+					require.Equal(t, msg.CgrpidTracker, id)
 
 					requireCgroupEventOpMkdir(t, msg, cgrpMapPath)
 					cgroupHierarchiesMap[controller][msg.CgrpData.Level-1].added = true
 					// Save the cgrpid of the cgroup_mkdir so we can match it later with cgrpid of execve
 					cgroupHierarchiesMap[controller][msg.CgrpData.Level-1].mkdirCgrpID = msg.CgrpidTracker
 				case ops.MSG_OP_CGROUP_RMDIR:
-					require.EqualValues(t, cgroupHierarchiesMap[controller][msg.CgrpData.Level-1].path, cgrpName)
+					require.Equal(t, cgroupHierarchiesMap[controller][msg.CgrpData.Level-1].path, cgrpName)
 					requireCgroupEventOpRmdir(t, msg, cgrpMapPath)
 					cgroupHierarchiesMap[controller][msg.CgrpData.Level-1].removed = true
 				}
 			}
 		}
 		if exec, ok := ev.(*grpcexec.MsgExecveEventUnix); ok {
-			if strings.Contains(exec.Process.Filename, "printf") {
-				args := strings.Split(exec.Process.Args, `\n`)
+			if strings.Contains(exec.Unix.Process.Filename, "printf") {
+				args := strings.Split(exec.Unix.Process.Args, `\n`)
 				argDir := args[0]
 				for i, cgroup := range cgroupHierarchiesMap[selectedController] {
 					// Ensure that argument matches the target cgroup directory of
 					// the hierarchy we want
 					if cgroup.path == argDir {
 						// cgroup path match, let's save the cgrpid of the execve
-						cgroupHierarchiesMap[selectedController][i].execCgrpID = exec.Kube.Cgrpid
+						cgroupHierarchiesMap[selectedController][i].execCgrpID = exec.Unix.Msg.Kube.Cgrpid
 						// save the received docker id of the execve
-						cgroupHierarchiesMap[selectedController][i].actualDocker = exec.Kube.Docker
+						cgroupHierarchiesMap[selectedController][i].actualDocker = exec.Unix.Kube.Docker
 						// we had our match break
 						break
 					}
@@ -480,7 +475,7 @@ func assertCgroupv2Events(ctx context.Context, t *testing.T, cgroupRoot string,
 				switch op {
 				case ops.MSG_OP_CGROUP_MKDIR:
 					require.EqualValues(t, ops.CGROUP_NEW, msg.CgrpData.State)
-					require.EqualValues(t, cgroupHierarchy[msg.CgrpData.Level-1].path, cgrpName)
+					require.Equal(t, cgroupHierarchy[msg.CgrpData.Level-1].path, cgrpName)
 
 					// Get cgroup id from path
 					targetPath := filepath.Join(cgroupRoot, cgrpPath)
@@ -488,31 +483,31 @@ func assertCgroupv2Events(ctx context.Context, t *testing.T, cgroupRoot string,
 					require.NoErrorf(t, err, "failed to get cgroup ID from path %s", targetPath)
 					require.NoError(t, err)
 					// Assert that received cgroup id from event is same as the id from the cgroup fs
-					require.EqualValues(t, msg.CgrpidTracker, id)
+					require.Equal(t, msg.CgrpidTracker, id)
 
 					requireCgroupEventOpMkdir(t, msg, cgrpMapPath)
 					cgroupHierarchy[msg.CgrpData.Level-1].added = true
 					// Save the cgrpid of the cgroup_mkdir so we can match it later with cgrpid of execve
 					cgroupHierarchy[msg.CgrpData.Level-1].mkdirCgrpID = msg.CgrpidTracker
 				case ops.MSG_OP_CGROUP_RMDIR:
-					require.EqualValues(t, cgroupHierarchy[msg.CgrpData.Level-1].path, cgrpName)
+					require.Equal(t, cgroupHierarchy[msg.CgrpData.Level-1].path, cgrpName)
 					requireCgroupEventOpRmdir(t, msg, cgrpMapPath)
 					cgroupHierarchy[msg.CgrpData.Level-1].removed = true
 				}
 			}
 		}
 		if exec, ok := ev.(*grpcexec.MsgExecveEventUnix); ok {
-			if strings.Contains(exec.Process.Filename, "printf") {
-				args := strings.Split(exec.Process.Args, `\n`)
+			if strings.Contains(exec.Unix.Process.Filename, "printf") {
+				args := strings.Split(exec.Unix.Process.Args, `\n`)
 				argDir := args[0]
 				for i, cgroup := range cgroupHierarchy {
 					// Ensure that argument matches the target cgroup directory of
 					// the hierarchy we want
 					if cgroup.path == argDir {
 						// cgroup path match, let's save the cgrpid of the execve
-						cgroupHierarchy[i].execCgrpID = exec.Kube.Cgrpid
+						cgroupHierarchy[i].execCgrpID = exec.Unix.Msg.Kube.Cgrpid
 						// save the received docker id of the execve
-						cgroupHierarchy[i].actualDocker = exec.Kube.Docker
+						cgroupHierarchy[i].actualDocker = exec.Unix.Kube.Docker
 						// we had our match break
 						break
 					}
@@ -578,7 +573,7 @@ func setupTgRuntimeConf(t *testing.T, trackingCgrpLevel, logLevel, hierarchyId,
 	}
 
 	if subSysIdx != invalidValue {
-		val.TgCgrpSubsysIdx = subSysIdx
+		val.TgCgrpv1SubsysIdx = subSysIdx
 	}
 
 	mapDir := bpf.MapPrefixPath()
@@ -588,78 +583,66 @@ func setupTgRuntimeConf(t *testing.T, trackingCgrpLevel, logLevel, hierarchyId,
 	}
 }
 
-func setupObserver(ctx context.Context, t *testing.T) *tus.TestSensorManager {
-	testManager := tus.StartTestSensorManager(ctx, t)
-	observer.SensorManager = testManager.Manager
-
-	if err := observer.InitDataCache(1024); err != nil {
-		t.Fatalf("failed to call observer.InitDataCache %s", err)
-	}
-	return testManager
-}
-
 // Test loading bpf cgroups programs
 func TestLoadCgroupsPrograms(t *testing.T) {
-	testutils.CaptureLog(t, logger.GetLogger().(*logrus.Logger))
+	testutils.CaptureLog(t, logger.GetLogger())
 
 	option.Config.HubbleLib = tus.Conf().TetragonLib
 	option.Config.Verbosity = 5
-	tus.LoadSensor(t, base.GetInitialSensor())
+	tus.LoadInitialSensor(t)
 	tus.LoadSensor(t, testsensor.GetTestSensor())
 	tus.LoadSensor(t, testsensor.GetCgroupSensor())
 }
 
 // Test `tg_conf_map` BPF map that it can hold runtime configuration
 func TestTgRuntimeConf(t *testing.T) {
-	testutils.CaptureLog(t, logger.GetLogger().(*logrus.Logger))
+	testutils.CaptureLog(t, logger.GetLogger())
 
 	option.Config.HubbleLib = tus.Conf().TetragonLib
 	option.Config.Verbosity = 5
 
-	tus.LoadSensor(t, base.GetInitialSensor())
+	tus.LoadInitialSensor(t)
 
 	val, err := testutils.GetTgRuntimeConf()
-	assert.NoError(t, err)
+	require.NoError(t, err)
 
 	assert.NotZero(t, val.NSPID)
 	assert.NotZero(t, val.CgrpFsMagic)
 
 	mapDir := bpf.MapPrefixPath()
 	err = confmap.UpdateConfMap(mapDir, val)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 
 	ret, err := testutils.ReadTgRuntimeConf(mapDir)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 
-	assert.EqualValues(t, ret, val)
+	assert.Equal(t, ret, val)
 
 	assert.Equal(t, ret.TgCgrpHierarchy, cgroups.GetCgrpHierarchyID())
-	assert.Equal(t, ret.TgCgrpSubsysIdx, cgroups.GetCgrpSubsystemIdx())
-	assert.Equal(t, ret.LogLevel, uint32(logger.GetLogLevel()))
+	assert.Equal(t, ret.TgCgrpv1SubsysIdx, cgroups.GetCgrpv1SubsystemIdx())
+	assert.Equal(t, ret.LogLevel, uint32(logger.GetLogLevel(logger.GetLogger())))
 }
 
 // Test we do not receive any cgroup events from BPF side
 func TestCgroupNoEvents(t *testing.T) {
-	testutils.CaptureLog(t, logger.GetLogger().(*logrus.Logger))
+	testutils.CaptureLog(t, logger.GetLogger())
 	ctx, cancel := context.WithTimeout(context.Background(), defaultTimeout)
 	defer cancel()
 
 	option.Config.HubbleLib = tus.Conf().TetragonLib
 	option.Config.Verbosity = 5
 
-	tus.LoadSensor(t, base.GetInitialSensor())
-
-	testManager := setupObserver(ctx, t)
-
-	testManager.AddAndEnableSensors(ctx, t, loadedSensors)
+	tus.LoadInitialSensor(t)
+	tus.LoadSensor(t, testsensor.GetTestSensor())
+	tus.LoadSensor(t, testsensor.GetCgroupSensor())
 
 	// Set Cgroup Tracking level to Zero means no tracking and no
 	// cgroup events, all bpf cgroups related programs have no effect
 	trackingCgrpLevel := uint32(0)
-	setupTgRuntimeConf(t, trackingCgrpLevel, uint32(logrus.TraceLevel), invalidValue, invalidValue)
+	setupTgRuntimeConf(t, trackingCgrpLevel, tetragonTraceLevel, invalidValue, invalidValue)
 
 	cgroupFSPath := cgroups.GetCgroupFSPath()
-	assert.NotEmpty(t, cgroupFSPath)
+	require.NotEmpty(t, cgroupFSPath)
 
 	dir, hierarchy := getTestCgroupDirAndHierarchy(t)
 	cgroupRmdir(t, cgroupFSPath, hierarchy, tetragonCgrpRoot)
@@ -676,7 +659,7 @@ func TestCgroupNoEvents(t *testing.T) {
 
 	trigger := func() {
 		err = cgroupMkdir(t, cgroupFSPath, hierarchy, dir)
-		assert.NoError(t, err)
+		require.NoError(t, err)
 	}
 
 	events := perfring.RunTestEvents(t, ctx, trigger)
@@ -692,29 +675,24 @@ func TestCgroupNoEvents(t *testing.T) {
 
 // Ensure that we get cgroup_{mkdir|rmdir} events
 func TestCgroupEventMkdirRmdir(t *testing.T) {
-	testutils.CaptureLog(t, logger.GetLogger().(*logrus.Logger))
+	testutils.CaptureLog(t, logger.GetLogger())
 	ctx, cancel := context.WithTimeout(context.Background(), defaultTimeout)
 	defer cancel()
 
 	option.Config.HubbleLib = tus.Conf().TetragonLib
 	option.Config.Verbosity = 5
 
-	tus.LoadSensor(t, base.GetInitialSensor())
-
-	testManager := setupObserver(ctx, t)
-
-	testManager.AddAndEnableSensors(ctx, t, loadedSensors)
-	t.Cleanup(func() {
-		testManager.DisableSensors(ctx, t, loadedSensors)
-	})
+	tus.LoadInitialSensor(t)
+	tus.LoadSensor(t, testsensor.GetTestSensor())
+	tus.LoadSensor(t, testsensor.GetCgroupSensor())
 
 	// Set Tracking level to 3 so we receive notifcations about
 	// /sys/fs/cgroup/$1/$2/$3 all cgroups that are at level <=3
 	trackingCgrpLevel := uint32(3)
-	setupTgRuntimeConf(t, trackingCgrpLevel, uint32(logrus.TraceLevel), invalidValue, invalidValue)
+	setupTgRuntimeConf(t, trackingCgrpLevel, tetragonTraceLevel, invalidValue, invalidValue)
 
 	cgroupFSPath := cgroups.GetCgroupFSPath()
-	assert.NotEmpty(t, cgroupFSPath)
+	require.NotEmpty(t, cgroupFSPath)
 
 	dir, hierarchy := getTestCgroupDirAndHierarchy(t)
 	cgroupRmdir(t, cgroupFSPath, hierarchy, tetragonCgrpRoot)
@@ -732,10 +710,10 @@ func TestCgroupEventMkdirRmdir(t *testing.T) {
 
 	trigger := func() {
 		err = cgroupMkdir(t, cgroupFSPath, hierarchy, dir)
-		assert.NoError(t, err)
+		require.NoError(t, err)
 
 		err = cgroupRmdir(t, cgroupFSPath, hierarchy, dir)
-		assert.NoError(t, err)
+		require.NoError(t, err)
 	}
 
 	mkdir := false
@@ -750,21 +728,20 @@ func TestCgroupEventMkdirRmdir(t *testing.T) {
 				cgrpPath := cgroups.CgroupNameFromCStr(msg.Path[:processapi.CGROUP_PATH_LENGTH])
 				op := ops.CgroupOpCode(msg.CgrpOp)
 				st := ops.CgroupState(msg.CgrpData.State).String()
-				logger.GetLogger().WithFields(logrus.Fields{
-					"cgroup.event":     op.String(),
-					"PID":              msg.PID,
-					"NSPID":            msg.NSPID,
-					"cgroup.IDTracker": msg.CgrpidTracker,
-					"cgroup.ID":        msg.Cgrpid,
-					"cgroup.state":     st,
-					"cgroup.level":     msg.CgrpData.Level,
-					"cgroup.path":      cgrpPath,
-				}).Info("Received Cgroup event")
+				logger.GetLogger().Info("Received Cgroup event",
+					"cgroup.event", op.String(),
+					"PID", msg.PID,
+					"NSPID", msg.NSPID,
+					"cgroup.IDTracker", msg.CgrpidTracker,
+					"cgroup.ID", msg.Cgrpid,
+					"cgroup.state", st,
+					"cgroup.level", msg.CgrpData.Level,
+					"cgroup.path", cgrpPath)
 
 				assert.NotZero(t, msg.PID)
 				assert.NotZero(t, msg.Cgrpid)
 				assert.NotZero(t, msg.CgrpidTracker)
-				assert.NotEqualValues(t, msg.CgrpData.State, ops.CGROUP_UNTRACKED)
+				assert.NotEqualValues(t, ops.CGROUP_UNTRACKED, msg.CgrpData.State)
 				assert.NotZero(t, msg.CgrpData.Level)
 
 				switch op {
@@ -773,7 +750,7 @@ func TestCgroupEventMkdirRmdir(t *testing.T) {
 					// Match only our test
 					if cgrpPath == matchedPath {
 						cgrpName := cgroups.CgroupNameFromCStr(msg.CgrpData.Name[:processapi.CGROUP_NAME_LENGTH])
-						assert.EqualValues(t, t.Name(), cgrpName)
+						assert.Equal(t, t.Name(), cgrpName)
 
 						mkdir = true
 						cgrpTrackingId = msg.CgrpidTracker
@@ -782,7 +759,7 @@ func TestCgroupEventMkdirRmdir(t *testing.T) {
 					// Match only our test
 					if cgrpPath == matchedPath {
 						cgrpName := cgroups.CgroupNameFromCStr(msg.CgrpData.Name[:processapi.CGROUP_NAME_LENGTH])
-						assert.EqualValues(t, t.Name(), cgrpName)
+						assert.Equal(t, t.Name(), cgrpName)
 						rmdir = true
 					}
 				}
@@ -791,13 +768,13 @@ func TestCgroupEventMkdirRmdir(t *testing.T) {
 	}
 
 	// Ensure that we received proper events
-	assert.Equal(t, true, mkdir)
-	assert.Equal(t, true, rmdir)
-	assert.NotZero(t, true, cgrpTrackingId)
+	assert.True(t, mkdir)
+	assert.True(t, rmdir)
+	assert.NotZero(t, cgrpTrackingId)
 
 	// Should be removed from the tracking map
 	_, err = cgrouptrackmap.LookupTrackingCgroup(cgrpMapPath, cgrpTrackingId)
-	assert.Error(t, err)
+	require.Error(t, err)
 }
 
 func testCgroupv2HierarchyInHybrid(ctx context.Context, t *testing.T,
@@ -880,13 +857,6 @@ func testCgroupv2HierarchyInUnified(ctx context.Context, t *testing.T,
 // Test Cgroupv2 tries to emulate k8s hierarchy without exec context
 // Works in systemd unified and hybrid mode according to parameter
 func testCgroupv2K8sHierarchy(ctx context.Context, t *testing.T, mode cgroups.CgroupModeCode, withExec bool) {
-	testManager := setupObserver(ctx, t)
-
-	testManager.AddAndEnableSensors(ctx, t, loadedSensors)
-	t.Cleanup(func() {
-		testManager.DisableSensors(ctx, t, loadedSensors)
-	})
-
 	_, err := testutils.GetTgRuntimeConf()
 	require.NoError(t, err)
 	if mode != cgroups.CGROUP_HYBRID && mode != cgroups.CGROUP_UNIFIED {
@@ -906,15 +876,15 @@ func testCgroupv2K8sHierarchy(ctx context.Context, t *testing.T, mode cgroups.Cg
 
 	trackingCgrpLevel := getTrackingLevel(defaultKubeCgroupHierarchy)
 	require.NotZero(t, trackingCgrpLevel)
-	require.True(t, trackingCgrpLevel <= uint32(len(defaultKubeCgroupHierarchy)))
+	require.LessOrEqual(t, trackingCgrpLevel, uint32(len(defaultKubeCgroupHierarchy)))
 
 	// Setup unified cgroup tracking 0 as hierarchy ID
-	setupTgRuntimeConf(t, trackingCgrpLevel, uint32(logrus.TraceLevel), 0, invalidValue)
+	setupTgRuntimeConf(t, trackingCgrpLevel, tetragonTraceLevel, 0, invalidValue)
 
 	logDefaultCgroupConfig(t)
 	logTetragonConfig(t, bpf.MapPrefixPath())
 	err = logCgroupMountInfo(t)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 
 	kubeCgroupHierarchy := make([]cgroupHierarchy, 0)
 	for i, c := range defaultKubeCgroupHierarchy {
@@ -952,7 +922,7 @@ func testCgroupv2K8sHierarchy(ctx context.Context, t *testing.T, mode cgroups.Cg
 			path = filepath.Join(path, dir.path)
 		}
 
-		for i := 0; i < len(kubeCgroupHierarchy); i++ {
+		for range kubeCgroupHierarchy {
 			err := cgroupRmdir(t, cgroupRoot, "", path)
 			if err != nil {
 				t.Fatalf("Failed to remove cgroup %s/%s", cgroupRoot, path)
@@ -1009,14 +979,15 @@ func testCgroupv2K8sHierarchy(ctx context.Context, t *testing.T, mode cgroups.Cg
 		testCgroupv2HierarchyInUnified(ctx, t, cgroupRoot, kubeCgroupHierarchy,
 			trackingCgrpLevel, triggersExecIDs)
 	} else {
-		if mode == cgroups.CGROUP_HYBRID {
+		switch mode {
+		case cgroups.CGROUP_HYBRID:
 			// This test will run in hybrid mode since systemd will mount first cgroup2
 			testCgroupv2HierarchyInHybrid(ctx, t, cgroupRoot, kubeCgroupHierarchy,
 				trackingCgrpLevel, triggersMkdirRmdir)
-		} else if mode == cgroups.CGROUP_UNIFIED {
+		case cgroups.CGROUP_UNIFIED:
 			testCgroupv2HierarchyInUnified(ctx, t, cgroupRoot, kubeCgroupHierarchy,
 				trackingCgrpLevel, triggersMkdirRmdir)
-		} else {
+		default:
 			t.Fatalf("Test %s unsupported Cgroup Mode", t.Name())
 		}
 	}
@@ -1043,14 +1014,16 @@ func testCgroupv2K8sHierarchy(ctx context.Context, t *testing.T, mode cgroups.Cg
 // Test Cgroupv2 tries to emulate k8s hierarchy without exec context
 // Works in systemd Unified pure cgroupv2
 func TestCgroupv2K8sHierarchyInUnified(t *testing.T) {
-	testutils.CaptureLog(t, logger.GetLogger().(*logrus.Logger))
+	testutils.CaptureLog(t, logger.GetLogger())
 	ctx, cancel := context.WithTimeout(context.Background(), defaultTimeout)
 	defer cancel()
 
 	option.Config.HubbleLib = tus.Conf().TetragonLib
 	option.Config.Verbosity = 5
 
-	tus.LoadSensor(t, base.GetInitialSensor())
+	tus.LoadInitialSensor(t)
+	tus.LoadSensor(t, testsensor.GetTestSensor())
+	tus.LoadSensor(t, testsensor.GetCgroupSensor())
 
 	// Probe full environment detection
 	setupTgRuntimeConf(t, invalidValue, invalidValue, invalidValue, invalidValue)
@@ -1065,14 +1038,14 @@ func TestCgroupv2K8sHierarchyInUnified(t *testing.T) {
 // Test Cgroupv2 tries to emulate k8s hierarchy without exec context
 // Works in systemd hybrid mode
 func TestCgroupv2K8sHierarchyInHybrid(t *testing.T) {
-	testutils.CaptureLog(t, logger.GetLogger().(*logrus.Logger))
+	testutils.CaptureLog(t, logger.GetLogger())
 	ctx, cancel := context.WithTimeout(context.Background(), defaultTimeout)
 	defer cancel()
 
 	option.Config.HubbleLib = tus.Conf().TetragonLib
 	option.Config.Verbosity = 5
 
-	tus.LoadSensor(t, base.GetInitialSensor())
+	tus.LoadInitialSensor(t)
 
 	// Probe full environment detection
 	setupTgRuntimeConf(t, invalidValue, invalidValue, invalidValue, invalidValue)
@@ -1085,18 +1058,16 @@ func TestCgroupv2K8sHierarchyInHybrid(t *testing.T) {
 }
 
 func testCgroupv1K8sHierarchyInHybrid(t *testing.T, withExec bool, selectedController string) {
-	testutils.CaptureLog(t, logger.GetLogger().(*logrus.Logger))
+	testutils.CaptureLog(t, logger.GetLogger())
 	ctx, cancel := context.WithTimeout(context.Background(), defaultTimeout)
 	defer cancel()
 
 	option.Config.HubbleLib = tus.Conf().TetragonLib
 	option.Config.Verbosity = 5
 
-	tus.LoadSensor(t, base.GetInitialSensor())
-
-	testManager := setupObserver(ctx, t)
-
-	testManager.AddAndEnableSensors(ctx, t, loadedSensors)
+	tus.LoadInitialSensor(t)
+	tus.LoadSensor(t, testsensor.GetTestSensor())
+	tus.LoadSensor(t, testsensor.GetCgroupSensor())
 
 	// Probe full environment detection
 	_, err := testutils.GetTgRuntimeConf()
@@ -1118,16 +1089,16 @@ func testCgroupv1K8sHierarchyInHybrid(t *testing.T, withExec bool, selectedContr
 
 	trackingCgrpLevel := getTrackingLevel(defaultKubeCgroupHierarchy)
 	require.NotZero(t, trackingCgrpLevel)
-	require.True(t, trackingCgrpLevel <= uint32(len(defaultKubeCgroupHierarchy)))
+	require.LessOrEqual(t, trackingCgrpLevel, uint32(len(defaultKubeCgroupHierarchy)))
 
 	// First setup default cgroup with our tracking level and trace level
-	setupTgRuntimeConf(t, trackingCgrpLevel, uint32(logrus.TraceLevel), invalidValue, invalidValue)
+	setupTgRuntimeConf(t, trackingCgrpLevel, tetragonTraceLevel, invalidValue, invalidValue)
 	// Fetch default controller name that we will use
 	usedController := cgroups.GetCgrpControllerName()
 
 	// See if we should use another controller for testing
 	if selectedController != usedController {
-		usedController = changeTestCgrpController(t, trackingCgrpLevel, uint32(logrus.TraceLevel), selectedController)
+		usedController = changeTestCgrpController(t, trackingCgrpLevel, tetragonTraceLevel, selectedController)
 		if selectedController == "memory" || selectedController == "pids" {
 			// We should always succeed to use memory or pids controllers otherwise panic
 			require.NotEmptyf(t, usedController, "failed to use the %s controller", selectedController)
@@ -1158,7 +1129,7 @@ func testCgroupv1K8sHierarchyInHybrid(t *testing.T, withExec bool, selectedContr
 	logDefaultCgroupConfig(t)
 	logTetragonConfig(t, bpf.MapPrefixPath())
 	err = logCgroupMountInfo(t)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 
 	triggerCgroupMkdir := func() {
 		for hierarchy, cgroupHierarchy := range kubeCgroupHierarchiesMap {
@@ -1180,7 +1151,7 @@ func testCgroupv1K8sHierarchyInHybrid(t *testing.T, withExec bool, selectedContr
 				path = filepath.Join(path, dir.path)
 			}
 
-			for i := 0; i < len(cgroupHierarchy); i++ {
+			for range cgroupHierarchy {
 				err := cgroupRmdir(t, cgroupRoot, hierarchy, path)
 				if err != nil {
 					t.Fatalf("Failed to remove cgroup %s/%s/%s", cgroupRoot, hierarchy, path)
@@ -1233,7 +1204,7 @@ func testCgroupv1K8sHierarchyInHybrid(t *testing.T, withExec bool, selectedContr
 		for _, dir := range kubeCgroupHierarchiesMap[usedController] {
 			path = filepath.Join(path, dir.path)
 		}
-		for i := 0; i < len(kubeCgroupHierarchiesMap[usedController]); i++ {
+		for range len(kubeCgroupHierarchiesMap[usedController]) {
 			err := cgroupRmdir(t, cgroupRoot, usedController, path)
 			if err != nil {
 				t.Fatalf("Failed to remove cgroup %s/%s/%s", cgroupRoot, usedController, path)
@@ -1322,7 +1293,7 @@ func TestCgroupv1ExecK8sHierarchyInHybridMemory(t *testing.T) {
 	testCgroupv1K8sHierarchyInHybrid(t, true, "memory")
 }
 
-// This test will use the piDisableSensorsds cgroup controller if available
+// This test will use the pids cgroup controller if available
 func TestCgroupv1ExecK8sHierarchyInHybridPids(t *testing.T) {
 	testCgroupv1K8sHierarchyInHybrid(t, true, "pids")
 }
@@ -1342,14 +1313,16 @@ func TestCgroupv1ExecK8sHierarchyInHybridInvalid(t *testing.T) {
 // Test Cgroupv2 tries to emulate k8s hierarchy with exec context
 // Works in systemd hybrid mode
 func TestCgroupv2ExecK8sHierarchyInUnified(t *testing.T) {
-	testutils.CaptureLog(t, logger.GetLogger().(*logrus.Logger))
+	testutils.CaptureLog(t, logger.GetLogger())
 	ctx, cancel := context.WithTimeout(context.Background(), defaultTimeout)
 	defer cancel()
 
 	option.Config.HubbleLib = tus.Conf().TetragonLib
 	option.Config.Verbosity = 5
 
-	tus.LoadSensor(t, base.GetInitialSensor())
+	tus.LoadInitialSensor(t)
+	tus.LoadSensor(t, testsensor.GetTestSensor())
+	tus.LoadSensor(t, testsensor.GetCgroupSensor())
 
 	// Probe full environment detection
 	_, err := testutils.GetTgRuntimeConf()
