@@ -1,3 +1,8 @@
+// SPDX-License-Identifier: Apache-2.0
+// Copyright Authors of Tetragon
+
+//go:build !windows
+
 package cgroup
 
 import (
@@ -16,15 +21,15 @@ var (
 )
 
 func LoadSockOpt(
-	bpfDir, mapDir, ciliumDir string,
-	load *program.Program, verbose int,
+	bpfDir string,
+	load *program.Program, maps []*program.Map, verbose int,
 ) error {
-	return LoadCgroupProgram(bpfDir, mapDir, ciliumDir, load, verbose)
+	return LoadCgroupProgram(bpfDir, load, maps, verbose)
 }
 
 func LoadCgroupProgram(
-	bpfDir, mapDir, ciliumDir string,
-	load *program.Program, verbose int) error {
+	bpfDir string,
+	load *program.Program, maps []*program.Map, verbose int) error {
 	if fgsCgroupFD < 0 {
 		fd, err := unix.Open(fgsCgroupPath, unix.O_RDONLY, 0)
 		if err != nil {
@@ -32,5 +37,5 @@ func LoadCgroupProgram(
 		}
 		fgsCgroupFD = fd
 	}
-	return program.LoadProgram(bpfDir, []string{mapDir, ciliumDir}, load, program.RawAttach(fgsCgroupFD), verbose)
+	return program.LoadProgram(bpfDir, load, maps, program.RawAttach(fgsCgroupFD), verbose)
 }
