@@ -1,13 +1,19 @@
 // SPDX-License-Identifier: Apache-2.0
 // Copyright Authors of Tetragon
 
+//go:build !windows
+
 package tracing
 
 import (
 	"context"
+	"errors"
 	"fmt"
+	"log/slog"
 	"os"
 	"os/exec"
+	"strconv"
+	"strings"
 	"sync"
 	"syscall"
 	"testing"
@@ -16,20 +22,24 @@ import (
 	"github.com/cilium/ebpf"
 	"github.com/cilium/tetragon/api/v1/tetragon"
 	ec "github.com/cilium/tetragon/api/v1/tetragon/codegen/eventchecker"
+	"github.com/cilium/tetragon/pkg/config"
+	"github.com/cilium/tetragon/pkg/grpc/tracing"
 	"github.com/cilium/tetragon/pkg/jsonchecker"
 	"github.com/cilium/tetragon/pkg/k8s/apis/cilium.io/v1alpha1"
 	"github.com/cilium/tetragon/pkg/kernels"
 	"github.com/cilium/tetragon/pkg/logger"
 	lc "github.com/cilium/tetragon/pkg/matchers/listmatcher"
-	smatcher "github.com/cilium/tetragon/pkg/matchers/stringmatcher"
-	"github.com/cilium/tetragon/pkg/observer"
+	"github.com/cilium/tetragon/pkg/matchers/stringmatcher"
+	"github.com/cilium/tetragon/pkg/observer/observertesthelper"
 	"github.com/cilium/tetragon/pkg/policyfilter"
+	"github.com/cilium/tetragon/pkg/reader/notify"
 	"github.com/cilium/tetragon/pkg/sensors"
 	testsensor "github.com/cilium/tetragon/pkg/sensors/test"
 	"github.com/cilium/tetragon/pkg/testutils"
+	tuo "github.com/cilium/tetragon/pkg/testutils/observer"
+	"github.com/cilium/tetragon/pkg/testutils/perfring"
 	tus "github.com/cilium/tetragon/pkg/testutils/sensors"
-	"github.com/sirupsen/logrus"
-	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 	"golang.org/x/sys/unix"
 
 	_ "github.com/cilium/tetragon/pkg/sensors/exec"
@@ -53,7 +63,7 @@ func TestGenericTracepointSimple(t *testing.T) {
 	ctx, cancel := context.WithTimeout(context.Background(), tus.Conf().CmdWaitTime)
 	defer cancel()
 
-	lseekConf := GenericTracepointConf{
+	lseekConf := v1alpha1.TracepointSpec{
 		Subsystem: "syscalls",
 		Event:     "sys_enter_lseek",
 		Args: []v1alpha1.KProbeArg{
@@ -62,23 +72,30 @@ func TestGenericTracepointSimple(t *testing.T) {
 		},
 	}
 
+	spec := v1alpha1.TracingPolicySpec{
+		Tracepoints: []v1alpha1.TracepointSpec{lseekConf},
+		Lists:       []v1alpha1.ListSpec{},
+	}
+
 	// initialize observer
-	obs, err := observer.GetDefaultObserver(t, ctx, tus.Conf().TetragonLib)
+	obs, err := observertesthelper.GetDefaultObserver(t, ctx, tus.Conf().TetragonLib)
 	if err != nil {
 		t.Fatalf("GetDefaultObserver error: %s", err)
 	}
 
-	sm := tus.StartTestSensorManager(ctx, t)
+	sm := tuo.GetTestSensorManager(t)
 	// create and add sensor
-	sensor, err := createGenericTracepointSensor("GtpLseekTest", []GenericTracepointConf{lseekConf}, policyfilter.NoFilterID, "policyName")
+	policyInfo, err := newPolicyInfoFromSpec("", "policyName", policyfilter.NoFilterID, &spec, nil)
+	require.NoError(t, err)
+	sensor, err := createGenericTracepointSensor(&spec, "GtpLseekTest", policyInfo)
 	if err != nil {
 		t.Fatalf("failed to create generic tracepoint sensor: %s", err)
 	}
 	sm.AddAndEnableSensor(ctx, t, sensor, "GtpLseekTest")
 
 	tpChecker := ec.NewProcessTracepointChecker("").
-		WithSubsys(smatcher.Full("syscalls")).
-		WithEvent(smatcher.Full("sys_enter_lseek")).
+		WithSubsys(stringmatcher.Full("syscalls")).
+		WithEvent(stringmatcher.Full("sys_enter_lseek")).
 		WithArgs(ec.NewKprobeArgumentListMatcher().
 			WithOperator(lc.Ordered).
 			WithValues(
@@ -87,18 +104,18 @@ func TestGenericTracepointSimple(t *testing.T) {
 			))
 	checker := ec.NewUnorderedEventChecker(tpChecker)
 
-	observer.LoopEvents(ctx, t, &doneWG, &readyWG, obs)
+	observertesthelper.LoopEvents(ctx, t, &doneWG, &readyWG, obs)
 	readyWG.Wait()
 	unix.Seek(-1, 0, whenceBogusValue)
 	time.Sleep(1000 * time.Millisecond)
 	err = jsonchecker.JsonTestCheck(t, checker)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 }
 
 // doTestGenericTracepointPidFilter is a utility function for doing generic
 // tracepoint tests. It filters events based on the test program's pid, so that
 // we get more predictable results.
-func doTestGenericTracepointPidFilter(t *testing.T, conf GenericTracepointConf, selfOp func(), checkFn func(*tetragon.ProcessTracepoint) error) {
+func doTestGenericTracepointPidFilter(t *testing.T, conf v1alpha1.TracepointSpec, selfOp func(), checkFn func(*tetragon.ProcessTracepoint) error) {
 	if _, err := os.Stat("/sys/kernel/debug/tracing/events/syscalls"); os.IsNotExist(err) {
 		t.Skip("cannot use syscall tracepoints (consider enabling CONFIG_FTRACE_SYSCALLS)")
 	}
@@ -109,7 +126,7 @@ func doTestGenericTracepointPidFilter(t *testing.T, conf GenericTracepointConf,
 	ctx, cancel := context.WithTimeout(context.Background(), tus.Conf().CmdWaitTime)
 	defer cancel()
 
-	pid := int(observer.GetMyPid())
+	pid := int(observertesthelper.GetMyPid())
 	t.Logf("filtering for my pid (%d)", pid)
 	pidSelector := v1alpha1.PIDSelector{
 		Operator:       "In",
@@ -122,14 +139,21 @@ func doTestGenericTracepointPidFilter(t *testing.T, conf GenericTracepointConf,
 		conf.Selectors = make([]v1alpha1.KProbeSelector, 1)
 	}
 	conf.Selectors[0].MatchPIDs = append(conf.Selectors[0].MatchPIDs, pidSelector)
-	obs, err := observer.GetDefaultObserver(t, ctx, tus.Conf().TetragonLib)
+	obs, err := observertesthelper.GetDefaultObserver(t, ctx, tus.Conf().TetragonLib)
 	if err != nil {
 		t.Fatalf("GetDefaultObserver error: %s", err)
 	}
 
-	sm := tus.StartTestSensorManager(ctx, t)
+	spec := v1alpha1.TracingPolicySpec{
+		Tracepoints: []v1alpha1.TracepointSpec{conf},
+		Lists:       []v1alpha1.ListSpec{},
+	}
+
+	sm := tuo.GetTestSensorManager(t)
 	// create and add sensor
-	sensor, err := createGenericTracepointSensor("GtpLseekTest", []GenericTracepointConf{conf}, policyfilter.NoFilterID, "policyName")
+	policyInfo, err := newPolicyInfoFromSpec("", "policyName", policyfilter.NoFilterID, &spec, nil)
+	require.NoError(t, err)
+	sensor, err := createGenericTracepointSensor(&spec, "GtpLseekTest", policyInfo)
 	if err != nil {
 		t.Fatalf("failed to create generic tracepoint sensor: %s", err)
 	}
@@ -137,14 +161,14 @@ func doTestGenericTracepointPidFilter(t *testing.T, conf GenericTracepointConf,
 	testSensor := testsensor.GetTestSensor()
 	sm.AddAndEnableSensor(ctx, t, testSensor, "testSensor")
 
-	observer.LoopEvents(ctx, t, &doneWG, &readyWG, obs)
+	observertesthelper.LoopEvents(ctx, t, &doneWG, &readyWG, obs)
 	readyWG.Wait()
 	selfOp()
 	testsensor.TestCheckerMarkEnd(t)
 	t.Log("Marked test end")
 
 	tpEventsNr := 0
-	nextCheck := func(event ec.Event, l *logrus.Logger) (bool, error) {
+	nextCheck := func(event ec.Event, _ *slog.Logger) (bool, error) {
 		switch tpEvent := event.(type) {
 		case *tetragon.ProcessTracepoint:
 			if err := checkFn(tpEvent); err != nil {
@@ -161,7 +185,7 @@ func doTestGenericTracepointPidFilter(t *testing.T, conf GenericTracepointConf,
 
 		}
 	}
-	finalCheck := func(l *logrus.Logger) error {
+	finalCheck := func(_ *slog.Logger) error {
 		defer func() { tpEventsNr = 0 }()
 		// NB: in some cases we get more than one events. I think this
 		// might be due to -EINTR or similar return values.
@@ -184,7 +208,7 @@ func doTestGenericTracepointPidFilter(t *testing.T, conf GenericTracepointConf,
 }
 
 func TestGenericTracepointPidFilterLseek(t *testing.T) {
-	tracepointConf := GenericTracepointConf{
+	tracepointConf := v1alpha1.TracepointSpec{
 		Subsystem: "syscalls",
 		Event:     "sys_enter_lseek",
 	}
@@ -194,7 +218,7 @@ func TestGenericTracepointPidFilterLseek(t *testing.T) {
 		unix.Seek(-1, 0, whenceBogusValue)
 	}
 
-	check := func(event *tetragon.ProcessTracepoint) error {
+	check := func(_ *tetragon.ProcessTracepoint) error {
 		return nil
 	}
 
@@ -205,10 +229,10 @@ func TestGenericTracepointArgFilterLseek(t *testing.T) {
 	fd_u := int32(100)
 	fd := 100
 	whence_u := uint64(whenceBogusValue)
-	whenceStr := fmt.Sprintf("%d", whenceBogusValue)
+	whenceStr := strconv.Itoa(whenceBogusValue)
 	whence := whenceBogusValue
 
-	tracepointConf := GenericTracepointConf{
+	tracepointConf := v1alpha1.TracepointSpec{
 		Subsystem: "syscalls",
 		Event:     "sys_enter_lseek",
 		Args: []v1alpha1.KProbeArg{
@@ -268,10 +292,10 @@ func TestGenericTracepointMeta(t *testing.T) {
 	// We want to write to a file so we can filter by non-stdout fd and thus avoid
 	// catching all the writes to test logs
 	fd, err := syscall.Open("/tmp/testificate", syscall.O_CREAT|syscall.O_WRONLY, 0o644)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	defer func() { syscall.Unlink("/tmp/testificate") }()
 
-	tracepointConf := GenericTracepointConf{
+	tracepointConf := v1alpha1.TracepointSpec{
 		Subsystem: "syscalls",
 		Event:     "sys_enter_write",
 		Args: []v1alpha1.KProbeArg{
@@ -288,7 +312,7 @@ func TestGenericTracepointMeta(t *testing.T) {
 			MatchArgs: []v1alpha1.ArgSelector{{
 				Index:    5,
 				Operator: "eq",
-				Values:   []string{fmt.Sprint(fd)},
+				Values:   []string{strconv.Itoa(fd)},
 			}},
 		}},
 	}
@@ -313,7 +337,7 @@ func TestGenericTracepointMeta(t *testing.T) {
 		}
 		arg1 := string(arg1_.BytesArg)
 		if arg1 != "hello world" {
-			return fmt.Errorf("Arg does not match \"hello world\"")
+			return errors.New("Arg does not match \"hello world\"")
 		}
 		return nil
 	}
@@ -336,14 +360,14 @@ func TestGenericTracepointMeta(t *testing.T) {
 //
 // print fmt: "NR %ld (%lx, %lx, %lx, %lx, %lx, %lx)", REC->id, REC->args[0], REC->args[1], REC->args[2], REC->args[3], REC->args[4], REC->args[5]
 func TestGenericTracepointRawSyscall(t *testing.T) {
-	tracepointConf := GenericTracepointConf{
+	tracepointConf := v1alpha1.TracepointSpec{
 		Subsystem: "raw_syscalls",
 		Event:     "sys_enter",
 		Args: []v1alpha1.KProbeArg{
-			v1alpha1.KProbeArg{
+			{
 				Index: 4, /* id */
 			},
-			v1alpha1.KProbeArg{
+			{
 				Index: 5, /* args */
 			},
 		},
@@ -353,7 +377,7 @@ func TestGenericTracepointRawSyscall(t *testing.T) {
 					{
 						Index:    4,
 						Operator: "Equal",
-						Values:   []string{fmt.Sprintf("%d", unix.SYS_LSEEK)},
+						Values:   []string{strconv.Itoa(unix.SYS_LSEEK)},
 					},
 				},
 			},
@@ -390,7 +414,7 @@ func TestGenericTracepointRawSyscall(t *testing.T) {
 
 		var err error
 		args := make([]uint64, 3)
-		for i := 0; i < 3; i++ {
+		for i := range 3 {
 			args[i], err = getSizeArg(i + 1)
 			if err != nil {
 				return err
@@ -409,51 +433,50 @@ func TestGenericTracepointRawSyscall(t *testing.T) {
 
 func TestLoadTracepointSensor(t *testing.T) {
 	var sensorProgs = []tus.SensorProg{
-		0:  tus.SensorProg{Name: "generic_tracepoint_event", Type: ebpf.TracePoint},
-		1:  tus.SensorProg{Name: "generic_tracepoint_arg1", Type: ebpf.TracePoint},
-		2:  tus.SensorProg{Name: "generic_tracepoint_arg2", Type: ebpf.TracePoint},
-		3:  tus.SensorProg{Name: "generic_tracepoint_arg3", Type: ebpf.TracePoint},
-		4:  tus.SensorProg{Name: "generic_tracepoint_arg4", Type: ebpf.TracePoint},
-		5:  tus.SensorProg{Name: "generic_tracepoint_arg5", Type: ebpf.TracePoint},
-		6:  tus.SensorProg{Name: "generic_tracepoint_event0", Type: ebpf.TracePoint},
-		7:  tus.SensorProg{Name: "generic_tracepoint_event1", Type: ebpf.TracePoint},
-		8:  tus.SensorProg{Name: "generic_tracepoint_event2", Type: ebpf.TracePoint},
-		9:  tus.SensorProg{Name: "generic_tracepoint_event3", Type: ebpf.TracePoint},
-		10: tus.SensorProg{Name: "generic_tracepoint_event4", Type: ebpf.TracePoint},
-		11: tus.SensorProg{Name: "generic_tracepoint_filter", Type: ebpf.TracePoint},
-		12: tus.SensorProg{Name: "generic_tracepoint_actions", Type: ebpf.TracePoint},
-		13: tus.SensorProg{Name: "generic_tracepoint_output", Type: ebpf.TracePoint},
+		0: {Name: "generic_tracepoint_event", Type: ebpf.TracePoint},
+		1: {Name: "generic_tracepoint_arg", Type: ebpf.TracePoint},
+		2: {Name: "generic_tracepoint_process_event", Type: ebpf.TracePoint},
+		3: {Name: "generic_tracepoint_filter", Type: ebpf.TracePoint},
+		4: {Name: "generic_tracepoint_actions", Type: ebpf.TracePoint},
+		5: {Name: "generic_tracepoint_output", Type: ebpf.TracePoint},
 	}
 
 	var sensorMaps = []tus.SensorMap{
 		// all programs
-		tus.SensorMap{Name: "tp_heap", Progs: []uint{0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13}},
+		{Name: "process_call_heap", Progs: []uint{0, 1, 2, 3, 4, 5}},
 
 		// all but generic_tracepoint_output
-		tus.SensorMap{Name: "tp_calls", Progs: []uint{0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12}},
-
-		// only generic_tracepoint_event*
-		tus.SensorMap{Name: "buffer_heap_map", Progs: []uint{6, 7, 8, 9, 10}},
+		{Name: "tp_calls", Progs: []uint{0, 1, 2, 3, 4}},
 
 		// all but generic_tracepoint_event,generic_tracepoint_filter
-		tus.SensorMap{Name: "retprobe_map", Progs: []uint{1, 2, 3, 4, 5, 6, 7, 8, 9, 10}},
+		{Name: "retprobe_map", Progs: []uint{1, 2}},
 
 		// generic_tracepoint_output
-		tus.SensorMap{Name: "tcpmon_map", Progs: []uint{13}},
+		{Name: "tcpmon_map", Progs: []uint{5}},
 
 		// all kprobe but generic_tracepoint_filter
-		tus.SensorMap{Name: "config_map", Progs: []uint{0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10}},
+		{Name: "config_map", Progs: []uint{0, 2}},
+
+		// generic_tracepoint_event
+		{Name: "tg_conf_map", Progs: []uint{0}},
 	}
 
-	if kernels.EnableLargeProgs() {
+	if config.EnableLargeProgs() {
 		// shared with base sensor
-		sensorMaps = append(sensorMaps, tus.SensorMap{Name: "execve_map", Progs: []uint{0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13}})
+		sensorMaps = append(sensorMaps, tus.SensorMap{Name: "execve_map", Progs: []uint{3, 4, 5}})
+
+		// generic_tracepoint_event*,generic_tracepoint_filter
+		sensorMaps = append(sensorMaps, tus.SensorMap{Name: "buffer_heap_map", Progs: []uint{2, 3}})
 	} else {
 		// shared with base sensor
-		sensorMaps = append(sensorMaps, tus.SensorMap{Name: "execve_map", Progs: []uint{0, 1, 2, 3, 4, 5, 11}})
+		sensorMaps = append(sensorMaps, tus.SensorMap{Name: "execve_map", Progs: []uint{3}})
+
+		// only generic_tracepoint_event*
+		sensorMaps = append(sensorMaps, tus.SensorMap{Name: "buffer_heap_map", Progs: []uint{2}})
 	}
 
 	readHook := `
+apiVersion: cilium.io/v1alpha1
 kind: TracingPolicy
 metadata:
   name: "raw-syscalls"
@@ -475,25 +498,29 @@ spec:
 	if err != nil {
 		t.Fatalf("writeFile(%s): err %s", testConfigFile, err)
 	}
-	sens, err = observer.GetDefaultSensorsWithFile(t, context.TODO(), testConfigFile, tus.Conf().TetragonLib)
+	sens, err = observertesthelper.GetDefaultSensorsWithFile(t, testConfigFile, tus.Conf().TetragonLib)
 	if err != nil {
 		t.Fatalf("GetDefaultObserverWithFile error: %s", err)
 	}
 
 	tus.CheckSensorLoad(sens, sensorMaps, sensorProgs, t)
 
-	sensors.UnloadAll()
+	sensi := make([]sensors.SensorIface, 0, len(sens))
+	for _, s := range sens {
+		sensi = append(sensi, s)
+	}
+	sensors.UnloadSensors(sensi)
 }
 
 func TestTracepointCloneThreads(t *testing.T) {
-	testutils.CaptureLog(t, logger.GetLogger().(*logrus.Logger))
+	testutils.CaptureLog(t, logger.GetLogger())
 	var doneWG, readyWG sync.WaitGroup
 	defer doneWG.Wait()
 
 	ctx, cancel := context.WithTimeout(context.Background(), tus.Conf().CmdWaitTime)
 	defer cancel()
 
-	lseekConf := GenericTracepointConf{
+	lseekConf := v1alpha1.TracepointSpec{
 		Subsystem: "syscalls",
 		Event:     "sys_enter_lseek",
 		Args: []v1alpha1.KProbeArg{
@@ -502,6 +529,11 @@ func TestTracepointCloneThreads(t *testing.T) {
 		},
 	}
 
+	spec := v1alpha1.TracingPolicySpec{
+		Tracepoints: []v1alpha1.TracepointSpec{lseekConf},
+		Lists:       []v1alpha1.ListSpec{},
+	}
+
 	testBinPath := "contrib/tester-progs/threads-tester"
 	testBin := testutils.RepoRootPath(testBinPath)
 	testCmd := exec.CommandContext(ctx, testBin, "--sensor", "tracepoint")
@@ -513,20 +545,22 @@ func TestTracepointCloneThreads(t *testing.T) {
 
 	// initialize observer
 	t.Logf("starting observer")
-	obs, err := observer.GetDefaultObserver(t, ctx, tus.Conf().TetragonLib)
+	obs, err := observertesthelper.GetDefaultObserver(t, ctx, tus.Conf().TetragonLib)
 	if err != nil {
 		t.Fatalf("GetDefaultObserver error: %s", err)
 	}
 
-	sm := tus.StartTestSensorManager(ctx, t)
+	sm := tuo.GetTestSensorManager(t)
 	// create and add sensor
-	sensor, err := createGenericTracepointSensor("GtpLseekTest", []GenericTracepointConf{lseekConf}, policyfilter.NoFilterID, "policyName")
+	policyInfo, err := newPolicyInfoFromSpec("", "policyName", policyfilter.NoFilterID, &spec, nil)
+	require.NoError(t, err)
+	sensor, err := createGenericTracepointSensor(&spec, "GtpLseekTest", policyInfo)
 	if err != nil {
 		t.Fatalf("failed to create generic tracepoint sensor: %s", err)
 	}
 	sm.AddAndEnableSensor(ctx, t, sensor, "GtpLseekTest")
 
-	observer.LoopEvents(ctx, t, &doneWG, &readyWG, obs)
+	observertesthelper.LoopEvents(ctx, t, &doneWG, &readyWG, obs)
 	readyWG.Wait()
 
 	cti := &testutils.ThreadTesterInfo{}
@@ -542,7 +576,7 @@ func TestTracepointCloneThreads(t *testing.T) {
 	cti.AssertPidsTids(t)
 
 	parentCheck := ec.NewProcessChecker().
-		WithBinary(smatcher.Suffix("threads-tester")).
+		WithBinary(stringmatcher.Suffix("threads-tester")).
 		WithPid(cti.ParentPid).
 		WithTid(cti.ParentTid)
 
@@ -553,13 +587,13 @@ func TestTracepointCloneThreads(t *testing.T) {
 		WithProcess(parentCheck)
 
 	child1Checker := ec.NewProcessChecker().
-		WithBinary(smatcher.Suffix("threads-tester")).
+		WithBinary(stringmatcher.Suffix("threads-tester")).
 		WithPid(cti.Child1Pid).
 		WithTid(cti.Child1Tid)
 
 	child1TpChecker := ec.NewProcessTracepointChecker("").
-		WithSubsys(smatcher.Full("syscalls")).
-		WithEvent(smatcher.Full("sys_enter_lseek")).
+		WithSubsys(stringmatcher.Full("syscalls")).
+		WithEvent(stringmatcher.Full("sys_enter_lseek")).
 		WithArgs(ec.NewKprobeArgumentListMatcher().
 			WithOperator(lc.Ordered).
 			WithValues(
@@ -568,13 +602,13 @@ func TestTracepointCloneThreads(t *testing.T) {
 			)).WithProcess(child1Checker).WithParent(parentCheck)
 
 	thread1Checker := ec.NewProcessChecker().
-		WithBinary(smatcher.Suffix("threads-tester")).
+		WithBinary(stringmatcher.Suffix("threads-tester")).
 		WithPid(cti.Thread1Pid).
 		WithTid(cti.Thread1Tid)
 
 	thread1TpChecker := ec.NewProcessTracepointChecker("").
-		WithSubsys(smatcher.Full("syscalls")).
-		WithEvent(smatcher.Full("sys_enter_lseek")).
+		WithSubsys(stringmatcher.Full("syscalls")).
+		WithEvent(stringmatcher.Full("sys_enter_lseek")).
 		WithArgs(ec.NewKprobeArgumentListMatcher().
 			WithOperator(lc.Ordered).
 			WithValues(
@@ -585,7 +619,7 @@ func TestTracepointCloneThreads(t *testing.T) {
 	checker := ec.NewUnorderedEventChecker(execCheck, child1TpChecker, thread1TpChecker, exitCheck)
 
 	err = jsonchecker.JsonTestCheck(t, checker)
-	assert.NoError(t, err)
+	require.NoError(t, err)
 }
 
 func TestTracepointForceType(t *testing.T) {
@@ -596,6 +630,7 @@ func TestTracepointForceType(t *testing.T) {
 	defer cancel()
 
 	lseekConfigHook_ := `
+apiVersion: cilium.io/v1alpha1
 kind: TracingPolicy
 metadata:
   name: "raw-syscalls"
@@ -603,14 +638,15 @@ spec:
   tracepoints:
   - subsystem: "syscalls"
     event: "sys_enter_lseek"
+    message: "System call lseek tracepoint test"
     # args: add both the syscall id, and the array with the arguments
     args:
     # fd argument
     - index: 5
-      type: "sint32"
+      type: "int32"
     # whence argument
     - index: 7
-      type: "sint32"
+      type: "int32"
 `
 
 	lseekConfigHook := []byte(lseekConfigHook_)
@@ -619,11 +655,11 @@ spec:
 		t.Fatalf("writeFile(%s): err %s", testConfigFile, err)
 	}
 
-	obs, err := observer.GetDefaultObserverWithFile(t, ctx, testConfigFile, tus.Conf().TetragonLib, observer.WithMyPid())
+	obs, err := observertesthelper.GetDefaultObserverWithFile(t, ctx, testConfigFile, tus.Conf().TetragonLib, observertesthelper.WithMyPid())
 	if err != nil {
 		t.Fatalf("GetDefaultObserverWithFile error: %s", err)
 	}
-	observer.LoopEvents(ctx, t, &doneWG, &readyWG, obs)
+	observertesthelper.LoopEvents(ctx, t, &doneWG, &readyWG, obs)
 	readyWG.Wait()
 
 	testBinPath := "contrib/tester-progs/threads-tester"
@@ -648,7 +684,7 @@ spec:
 	cti.AssertPidsTids(t)
 
 	parentCheck := ec.NewProcessChecker().
-		WithBinary(smatcher.Suffix("threads-tester")).
+		WithBinary(stringmatcher.Suffix("threads-tester")).
 		WithPid(cti.ParentPid).
 		WithTid(cti.ParentTid)
 
@@ -659,13 +695,14 @@ spec:
 		WithProcess(parentCheck)
 
 	child1Checker := ec.NewProcessChecker().
-		WithBinary(smatcher.Suffix("threads-tester")).
+		WithBinary(stringmatcher.Suffix("threads-tester")).
 		WithPid(cti.Child1Pid).
 		WithTid(cti.Child1Tid)
 
 	child1TpChecker := ec.NewProcessTracepointChecker("").
-		WithSubsys(smatcher.Full("syscalls")).
-		WithEvent(smatcher.Full("sys_enter_lseek")).
+		WithSubsys(stringmatcher.Full("syscalls")).
+		WithEvent(stringmatcher.Full("sys_enter_lseek")).
+		WithMessage(stringmatcher.Full("System call lseek tracepoint test")).
 		WithArgs(ec.NewKprobeArgumentListMatcher().
 			WithOperator(lc.Ordered).
 			WithValues(
@@ -674,13 +711,14 @@ spec:
 			)).WithProcess(child1Checker).WithParent(parentCheck)
 
 	thread1Checker := ec.NewProcessChecker().
-		WithBinary(smatcher.Suffix("threads-tester")).
+		WithBinary(stringmatcher.Suffix("threads-tester")).
 		WithPid(cti.Thread1Pid).
 		WithTid(cti.Thread1Tid)
 
 	thread1TpChecker := ec.NewProcessTracepointChecker("").
-		WithSubsys(smatcher.Full("syscalls")).
-		WithEvent(smatcher.Full("sys_enter_lseek")).
+		WithSubsys(stringmatcher.Full("syscalls")).
+		WithEvent(stringmatcher.Full("sys_enter_lseek")).
+		WithMessage(stringmatcher.Full("System call lseek tracepoint test")).
 		WithArgs(ec.NewKprobeArgumentListMatcher().
 			WithOperator(lc.Ordered).
 			WithValues(
@@ -691,5 +729,213 @@ spec:
 	checker := ec.NewUnorderedEventChecker(execCheck, child1TpChecker, thread1TpChecker, exitCheck)
 
 	err = jsonchecker.JsonTestCheck(t, checker)
-	assert.NoError(t, err)
+	require.NoError(t, err)
+}
+
+func TestStringTracepoint(t *testing.T) {
+	testutils.CaptureLog(t, logger.GetLogger())
+	ctx, cancel := context.WithTimeout(context.Background(), tus.Conf().CmdWaitTime)
+	defer cancel()
+
+	mypid := int(observertesthelper.GetMyPid())
+	t.Logf("filtering for my pid (%d)", mypid)
+
+	spec := &v1alpha1.TracingPolicySpec{
+		Tracepoints: []v1alpha1.TracepointSpec{{
+			Subsystem: "syscalls",
+			Event:     "sys_enter_mkdirat",
+			Args: []v1alpha1.KProbeArg{{
+				Index: 4,
+			}, {
+				Index: 6,
+				Type:  "string",
+			}},
+			/*
+				Selectors: []v1alpha1.KProbeSelector{{
+					MatchPIDs: []v1alpha1.PIDSelector{{
+						Operator:    "In",
+						FollowForks: true,
+						Values:      []uint32{uint32(mypid)},
+					}},
+				}},
+			*/
+		}},
+	}
+
+	loadGenericSensorTest(t, spec)
+	t0 := time.Now()
+	loadElapsed := time.Since(t0)
+	t.Logf("loading sensors took: %s\n", loadElapsed)
+
+	countPizza := 0
+	eventFn := func(ev notify.Message) error {
+		if tpEvent, ok := ev.(*tracing.MsgGenericTracepointUnix); ok {
+			if tpEvent.Event != "sys_enter_mkdirat" {
+				return fmt.Errorf("unexpected tracepoint event, %s:%s", tpEvent.Subsys, tpEvent.Event)
+			}
+			arg := tpEvent.Args[1].(string)
+			if strings.Contains(arg, "pizzaisthebest") {
+				countPizza++
+			}
+		}
+		return nil
+	}
+
+	ops := func() {
+		// NB: unix.Mkdir uses unix.Mkdirat, so make things explicit
+		unix.Mkdirat(unix.AT_FDCWD, "/tmp/pizzaisthebest", 0777)
+		os.Remove("/tmp/pizzaisthebest")
+	}
+
+	perfring.RunTest(t, ctx, ops, eventFn)
+	require.Equal(t, 1, countPizza, "expected events with 'pizzaisthebest'")
+}
+
+func testListSyscallsDupsRange(t *testing.T, checker *ec.UnorderedEventChecker, configHook string) {
+	var doneWG, readyWG sync.WaitGroup
+	defer doneWG.Wait()
+
+	ctx, cancel := context.WithTimeout(context.Background(), tus.Conf().CmdWaitTime)
+	defer cancel()
+
+	err := os.WriteFile(testConfigFile, []byte(configHook), 0644)
+	if err != nil {
+		t.Fatalf("writeFile(%s): err %s", testConfigFile, err)
+	}
+
+	obs, err := observertesthelper.GetDefaultObserverWithFile(t, ctx, testConfigFile, tus.Conf().TetragonLib, observertesthelper.WithMyPid())
+	if err != nil {
+		t.Fatalf("GetDefaultObserverWithFile error: %s", err)
+	}
+	observertesthelper.LoopEvents(ctx, t, &doneWG, &readyWG, obs)
+	readyWG.Wait()
+
+	for i := 9900; i < 9930; i++ {
+		syscall.Dup(i)
+	}
+
+	err = jsonchecker.JsonTestCheck(t, checker)
+	require.NoError(t, err)
+}
+
+func TestTracepointListSyscallDupsRange(t *testing.T) {
+	if !kernels.MinKernelVersion("5.3.0") {
+		t.Skip("TestCopyFd requires at least 5.3.0 version")
+	}
+	myPid := observertesthelper.GetMyPid()
+	pidStr := strconv.Itoa(int(myPid))
+	configHook := `
+apiVersion: cilium.io/v1alpha1
+kind: TracingPolicy
+metadata:
+  name: "sys-write"
+spec:
+  lists:
+  - name: "test"
+    type: "syscalls"
+    values:
+    - "sys_dup"
+  tracepoints:
+  - subsystem: "raw_syscalls"
+    event: "sys_enter"
+    args:
+    - index: 4
+      type: "syscall64"
+    - index: 5
+      type: "uint64"
+    selectors:
+    - matchPIDs:
+      - operator: In
+        followForks: true
+        isNamespacePID: false
+        values:
+        - ` + pidStr + `
+      matchArgs:
+      - index: 0
+        operator: "InMap"
+        values:
+        - "list:test"
+      - index: 1
+        operator: "InMap"
+        values:
+        - 9910:9920
+`
+
+	// The test hooks raw tracepoint and uses InMap operator to filter
+	// for dup syscall and for fd in range 9910 to 9920.
+
+	checker := ec.NewUnorderedEventChecker()
+
+	for i := 9910; i < 9920; i++ {
+		tpCheckerDup := ec.NewProcessTracepointChecker("").
+			WithArgs(ec.NewKprobeArgumentListMatcher().
+				WithOperator(lc.Ordered).
+				WithValues(
+					ec.NewKprobeArgumentChecker().WithSyscallId(mkSysIDChecker(t, unix.SYS_DUP)),
+					ec.NewKprobeArgumentChecker().WithSizeArg(uint64(i)),
+				))
+
+		checker.AddChecks(tpCheckerDup)
+	}
+
+	testListSyscallsDupsRange(t, checker, configHook)
+}
+
+func TestTracepointResolve(t *testing.T) {
+	if !kernels.MinKernelVersion("5.4") {
+		t.Skip("Test requires kernel 5.4+")
+	}
+
+	var doneWG, readyWG sync.WaitGroup
+	defer doneWG.Wait()
+
+	ctx, cancel := context.WithTimeout(context.Background(), tus.Conf().CmdWaitTime)
+	defer cancel()
+
+	hook := `
+apiVersion: cilium.io/v1alpha1
+kind: TracingPolicy
+metadata:
+  name: "rawtp"
+spec:
+  tracepoints:
+    - subsystem: "sched"
+      event: "sched_process_exec"
+      raw: true
+      args:
+        - index: 2
+          type: "file"
+          resolve: "file"
+`
+
+	createCrdFile(t, hook)
+
+	testNop := testutils.RepoRootPath("contrib/tester-progs/nop")
+
+	obs, err := observertesthelper.GetDefaultObserverWithFile(t, ctx, testConfigFile, tus.Conf().TetragonLib)
+	if err != nil {
+		t.Fatalf("GetDefaultObserverWithFile error: %s", err)
+	}
+	observertesthelper.LoopEvents(ctx, t, &doneWG, &readyWG, obs)
+	readyWG.Wait()
+
+	if err := exec.Command(testNop).Run(); err != nil {
+		fmt.Printf("Failed to execute test binary: %s\n", err)
+	}
+
+	tpChecker := ec.NewProcessTracepointChecker("").
+		WithSubsys(stringmatcher.Full("sched")).
+		WithEvent(stringmatcher.Full("sched_process_exec")).
+		WithArgs(ec.NewKprobeArgumentListMatcher().
+			WithOperator(lc.Ordered).
+			WithValues(
+				ec.NewKprobeArgumentChecker().WithFileArg(ec.NewKprobeFileChecker().
+					WithPath(stringmatcher.Full(testNop)),
+				),
+			))
+
+	checker := ec.NewUnorderedEventChecker(tpChecker)
+
+	err = jsonchecker.JsonTestCheck(t, checker)
+	require.NoError(t, err)
 }
