@@ -8,41 +8,24 @@ import (
 	"context"
 
 	"github.com/cilium/tetragon/api/v1/tetragon"
-	"github.com/cilium/tetragon/pkg/sensors"
 	"github.com/cilium/tetragon/pkg/tracingpolicy"
 )
 
 type FakeObserver struct{}
 
-func (f *FakeObserver) ListSensors(ctx context.Context) (*[]sensors.SensorStatus, error) {
-	return nil, nil
-}
-
-func (f *FakeObserver) EnableSensor(ctx context.Context, name string) error {
-	return nil
-}
-
-func (f *FakeObserver) DisableSensor(ctx context.Context, name string) error {
-	return nil
-}
-
-func (f *FakeObserver) GetSensorConfig(ctx context.Context, k string, v string) (string, error) {
-	return "", nil
-}
-
-func (f *FakeObserver) SetSensorConfig(ctx context.Context, name string, cfgkey string, cfgval string) error {
-	return nil
-}
-
-func (f *FakeObserver) GetTreeProto(ctx context.Context, tname string) (*tetragon.StackTraceNode, error) {
-	return nil, nil
-}
-
 func (f *FakeObserver) AddTracingPolicy(ctx context.Context, tp tracingpolicy.TracingPolicy) error {
 	return nil
 }
 
-func (f *FakeObserver) DelTracingPolicy(ctx context.Context, sensorName string) error {
+func (f *FakeObserver) DeleteTracingPolicy(ctx context.Context, sensorName string, sensorNamespace string) error {
+	return nil
+}
+
+func (f *FakeObserver) EnableTracingPolicy(ctx context.Context, sensorName string, sensorNamespace string) error {
+	return nil
+}
+
+func (f *FakeObserver) DisableTracingPolicy(ctx context.Context, sensorName string, sensorNamespace string) error {
 	return nil
 }
 
@@ -53,3 +36,7 @@ func (f *FakeObserver) RemoveSensor(ctx context.Context, sensorName string) erro
 func (f *FakeObserver) ListTracingPolicies(ctx context.Context) (*tetragon.ListTracingPoliciesResponse, error) {
 	return nil, nil
 }
+
+func (h *FakeObserver) ConfigureTracingPolicy(_ context.Context, _ *tetragon.ConfigureTracingPolicyRequest) error {
+	return nil
+}
