@@ -1,17 +1,23 @@
 // SPDX-License-Identifier: Apache-2.0
 // Copyright Authors of Tetragon
 
+//go:build !windows
+
 package syscallinfo
 
 import (
 	"testing"
 
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 	"golang.org/x/sys/unix"
 )
 
 func TestSycallInfo(t *testing.T) {
-	sysName := GetSyscallName(unix.SYS_BPF)
+	abi, err := DefaultABI()
+	require.NoError(t, err)
+	sysName, err := GetSyscallName(abi, unix.SYS_BPF)
+	require.NoError(t, err)
 	if sysName != "bpf" {
 		t.Fatalf("got unexpected syscall name: %s (expecting bpf)", sysName)
 	}
