@@ -4,43 +4,45 @@
 package testutils
 
 import (
-	"io"
+	"log/slog"
 	"strings"
 	"testing"
 
+	"github.com/cilium/tetragon/pkg/logger"
 	tus "github.com/cilium/tetragon/pkg/testutils/sensors"
-
-	"github.com/sirupsen/logrus"
 )
 
 type LogCapturer struct {
-	*testing.T
-	Log *logrus.Logger
+	TB  testing.TB
+	Log *slog.Logger
 }
 
 func (tl LogCapturer) Write(p []byte) (n int, err error) {
 	// Since we are calling T.Log() here, we want to avoid appending multiple "\n", so
 	// trim whatever was added by the inner logger.
 	s := strings.TrimRight(string(p), "\n")
-	tl.T.Log(s)
+	tl.TB.Log(s)
 	return len(s), nil
 }
 
-// CaptureLog redirects logrus output to testing.Log
-func CaptureLog(t *testing.T, l *logrus.Logger) {
+// CaptureLog redirects slog output to testing.Log
+func CaptureLog(tb testing.TB, l *slog.Logger) {
 	lc := &LogCapturer{
-		T:   t,
+		TB:  tb,
 		Log: l,
 	}
 
-	origOut := logrus.StandardLogger().Out
-	t.Cleanup(func() {
-		l.SetOutput(origOut)
+	originalLogger := logger.DefaultSlogLogger
+
+	tb.Cleanup(func() {
+		logger.DefaultSlogLogger = originalLogger
 	})
 
 	if tus.Conf().DisableTetragonLogs {
-		l.SetOutput(io.Discard)
+		newLogger := slog.New(logger.SlogNopHandler)
+		logger.DefaultSlogLogger = newLogger
 	} else {
-		l.SetOutput(lc)
+		newLogger := slog.New(slog.NewTextHandler(lc, nil))
+		logger.DefaultSlogLogger = newLogger
 	}
 }
