@@ -21,17 +21,17 @@ func TestTimer(t *testing.T) {
 	timer1.Start(time.Duration(100) * time.Millisecond)
 	time.Sleep(time.Duration(550) * time.Millisecond)
 	timer1.Stop()
-	assert.Equal(count, 5, "Tests simple timer (100ms interval)")
+	assert.Equal(5, count, "Tests simple timer (100ms interval)")
 	timer1.Start(time.Duration(1000) * time.Millisecond)
 	time.Sleep(time.Duration(1500) * time.Millisecond)
-	assert.Equal(count, 6, "Tests simple timer (1000ms interval)")
+	assert.Equal(6, count, "Tests simple timer (1000ms interval)")
 	timer1.Start(time.Duration(200) * time.Millisecond)
 	time.Sleep(time.Duration(300) * time.Millisecond)
 	timer1.Stop()
-	assert.Equal(count, 7, "Tests restart of timer")
+	assert.Equal(7, count, "Tests restart of timer")
 }
 
 func Worker() {
 	count++
-	logger.GetLogger().WithField("count", count).Info("Counting")
+	logger.GetLogger().Info("Counting", "count", count)
 }
