@@ -1,6 +1,8 @@
 // SPDX-License-Identifier: Apache-2.0
 // Copyright Authors of Tetragon
 
+//go:build !windows
+
 package state
 
 // Key is a key that we use to store specific state in a test's context.Context
@@ -13,6 +15,8 @@ var (
 	InstallOpts = Key{slug: "InstallOpts"}
 	// Key for storing a list of ports we forwarded for gRPC
 	GrpcForwardedPorts = Key{slug: "GrpcForwardedPorts"}
+	// Key for storing a list of forwarded connections for gRPC
+	GrpcForwardedConns = Key{slug: "GrpcForwardedConns"}
 	// Key for storing a list of ports we forwarded for prometheus metics
 	PromForwardedPorts = Key{slug: "PromForwardedPorts"}
 	// Key for storing a list of ports we forwarded for the pprof server
@@ -25,6 +29,4 @@ var (
 	ExportDir = Key{slug: "ExportDir"}
 	// Key for storing the cluster name
 	ClusterName = Key{slug: "ClusterName"}
-	// Key for storing test failure
-	TestFailure = Key{slug: "TestFailure"}
 )
